"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInteractionsRefactor = exports.isCampaignPresent = exports.getInteractionById = void 0;
const Knex = require("knex");
const knex_config_1 = require("../knex.config");
const interaction_model_1 = require("../../../models/interaction.model");
const log4js = require("log4js");
const moment = require("moment");
let sql = require('mssql');
let logger = log4js.getLogger('ServerScripts');
const callDB = Knex(knex_config_1.connections.call);
const interactionsTable = 'InteractionDetail ID';
const callInteractionDetail = 'CallInteractionDetail CID';
const idColumn = 'ID.Id';
const vccColumn = 'ID.VirtualCC';
const contactAddressColumn = 'ID.ContactId';
const numberColumn = 'CASE WHEN ID.Direction = \'E\' THEN CID.CallerId ELSE CID.CalledId END AS Number ';
const dispositionCodeColumn = 'ID.ManagementResult';
const durationTimeColumn = 'ID.AttentionTime';
const EndDateColumn = 'ID.EndDate';
const StartDateColumn = 'ID.StartDate';
const agentColumn = 'ID.FirstAgent';
const originColumn = 'ID.Direction';
const campaignColumn = 'ID.Campaign';
const getInteractionById = function (vcc, id, loadMultimedia = false) {
    logger.info('[call.ts::getInteractionById] Searching vcc|interaction: ' + vcc + '|' + id);
    return new Promise(function (resolve, reject) {
        callDB.select([idColumn, campaignColumn, agentColumn,
            StartDateColumn, EndDateColumn, durationTimeColumn,
            dispositionCodeColumn, contactAddressColumn, callDB.raw(numberColumn), originColumn
        ])
            .join(callDB.raw('CallInteractionDetail CID (NOLOCK) '), function () {
            this.on(function () {
                this.on('CID.Id', '=', 'ID.Id');
            });
        })
            .from(callDB.raw(interactionsTable + ' (NOLOCK) ')).where({
            'ID.VirtualCC': vcc,
            'ID.Id': id
        }).then(result => {
            if (result && result[0]) {
                const r = result[0];
                if (typeof r !== "undefined") {
                    let interaction = new interaction_model_1.Interaction(r['Id'], r['Campaign'], r['FirstAgent'], r['StartDate'], r['EndDate'], r['AttentionTime'], 'CALL', r['ManagementResult'], r['ContactId'], parseOrigin(r['Direction']));
                    interaction.number = r['Number'];
                    interaction.descriptionAgent = r['FirstAgent'];
                    interaction.descriptionCampaign = r['Campaign'];
                    if (loadMultimedia) {
                        getInteractionMultimediaPartsById(vcc, interaction)
                            .then(i => {
                            resolve(i);
                        }, err => {
                            reject(err);
                        });
                    }
                    else {
                        resolve(interaction);
                    }
                }
                else
                    resolve(null);
            }
            else
                resolve(null);
        }, error => {
            reject(error);
        });
    });
};
exports.getInteractionById = getInteractionById;
const getInteractionMultimediaPartsById = function (vcc, interaction) {
    return new Promise((resolve, reject) => callDB.select([
        'InteractionMultimediaParts.Id',
        'InteractionMultimediaParts.Section',
        'InteractionMultimediaParts.Part',
        'InteractionActorDetail.Actor',
        'InteractionActorDetail.ActorName',
        'InteractionMultimediaParts.Format',
        'InteractionActorDetail.FirstAgent',
        'InteractionMultimediaParts.ContentType'
    ])
        .from(callDB.raw('InteractionMultimediaParts (NOLOCK) '))
        .join('InteractionActorDetail', function () {
        this.on(function () {
            this.on('InteractionActorDetail.Id', '=', 'InteractionMultimediaParts.Id')
                .on('InteractionActorDetail.Section', '=', 'InteractionMultimediaParts.Section');
        });
    })
        .where({
        'InteractionMultimediaParts.Id': interaction.id
    })
        .orderBy('InteractionMultimediaParts.TmStmp', 'ASC')
        .then(result => {
        logger.info('[call::getInteractionMultimediaPartsById] Multimedia parts by id: ' + interaction.id + '|' + result.length);
        if (result) {
            logger.info('[call::getInteractionMultimediaPartsById] Agent\'s parts : ' + interaction.id + '|' + result.length);
            interaction.interactionPartGroups = [];
            interaction.agentName = result[0].ActorName;
            result.forEach(r => {
                if (["mp3", "wav", "ogg"].includes(r.Format)) {
                    if (r['ContentType'] === 'screenrecording')
                        interaction.hasVideo = true;
                    let group = interaction.interactionPartGroups.find(e => e.agentId === r['Actor']);
                    if (!group) {
                        group = new interaction_model_1.MultimediaPartGroup(r['Actor']);
                        interaction.interactionPartGroups.push(group);
                    }
                    let p = new interaction_model_1.MultimediaPart(r['Section'], r['Part'], '', r['Actor'], r['Format']);
                    group.parts.push(p);
                }
            });
        }
        resolve(interaction);
    }, error => {
        reject(error);
    }));
};
const isCampaignPresent = function (vcc, campaignId) {
    return new Promise(function (resolve, reject) {
        callDB.distinct('Campaign').from(callDB.raw(interactionsTable + ' (NOLOCK) ')).where({
            'ID.VirtualCC': vcc,
            'ID.Campaign': campaignId
        }).then(result => {
            if (result && result.length > 0) {
                resolve(true);
            }
            else
                resolve(false);
        }, error => {
            reject(error);
        });
    });
};
exports.isCampaignPresent = isCampaignPresent;
function parseOrigin(origin) {
    if (origin === 'S')
        return 'OUT';
    else if (origin === 'E')
        return 'IN';
    return '';
}
const getInteractionsRefactor = function (vcc, limit, params) {
    logger.info('[HistoricalData::getInteractionsRefactor] Making interactions search query to HistoricalData.');
    return new Promise(function (resolve, reject) {
        let builder = callDB
            .from(callDB.raw(interactionsTable + ' (NOLOCK)'))
            .leftJoin(callDB.raw('InteractionMultimediaParts IMP (NOLOCK)'), 'IMP.Id', 'ID.id')
            .leftJoin(callDB.raw('InteractionActorDetail IAD (NOLOCK)'), 'IAD.Id', 'ID.id')
            .where({
            'ID.VirtualCC': vcc,
            'ID.IsTaked': true
        })
            .whereBetween(EndDateColumn, [params.dateIni, params.dateFin]);
        if (params.campaigns && params.campaigns.length)
            builder = builder.whereIn(campaignColumn, params.campaigns);
        if (params.agents && params.agents.length)
            builder = builder.whereIn('IAD.Actor', params.agents);
        if (params.origins && params.origins.length)
            builder = builder.whereIn(originColumn, params.origins);
        if (params.minDuration)
            builder = builder.whereRaw('?? >= ?', [durationTimeColumn, params.minDuration]);
        if (params.maxDuration)
            builder = builder.whereRaw('?? <= ?', [durationTimeColumn, params.maxDuration]);
        if (params.results && params.results.length)
            builder = builder.whereIn(dispositionCodeColumn, params.results);
        if (params.multimediaFormats && params.multimediaFormats.length) {
            builder = builder.whereIn('IMP.Format', params.multimediaFormats);
        }
        let horaIni = moment(params.hourIni, 'HH:mm:ss');
        let horaFin = moment(params.hourFin, 'HH:mm:ss');
        if (horaIni.isSameOrBefore(horaFin)) {
            builder = builder.whereRaw('CONVERT(VARCHAR(8), ??, 108) BETWEEN ? AND ?', [EndDateColumn, params.hourIni, params.hourFin]);
        }
        else if (horaIni.isAfter(horaFin)) {
            builder = builder.whereRaw('CONVERT(VARCHAR(8), ??, 108) NOT BETWEEN ? AND ?', [EndDateColumn, params.hourFin, params.hourIni]);
        }
        let builderResult = builder.clone();
        let builderTotal = builder.clone();
        let selectResultRaw = '\'CALL\' AS interactionType, '
            + idColumn + ' AS id, '
            + campaignColumn + ' AS idCampaign, '
            + campaignColumn + ' AS descriptionCampaign, '
            + agentColumn + ' AS idAgent, '
            + agentColumn + ' AS descriptionAgent, '
            + StartDateColumn + ' AS dateIni, '
            + EndDateColumn + ' AS dateFin, '
            + durationTimeColumn + ' AS interactionTime, '
            + dispositionCodeColumn + ' AS dispositionCode, '
            + contactAddressColumn + ' AS address, '
            + 'CASE ' + originColumn + ' WHEN \'S\' THEN \'OUT\' WHEN \'E\' THEN \'IN\' ELSE \'\' END AS origin ';
        if (params.withSectionPart === true) {
            builderTotal.countDistinct(idColumn + ' as totalRecords')
                .sum(durationTimeColumn + ' as totalTime')
                .select(callDB.raw('SUM(CEILING(' + durationTimeColumn + ' * 1.00 /60) * 60) as roundedTime'));
            selectResultRaw += ', '
                + '[IMP].[Format] AS format, '
                + '[IMP].[Section] AS interactionSection, '
                + '[IMP].[Part] AS interactionPart ';
        }
        else {
            builderTotal.countDistinct(idColumn + ' as totalRecords')
                .sum(durationTimeColumn + ' as totalTime')
                .select(callDB.raw('SUM(CEILING(' + durationTimeColumn + ' * 1.00 /60) * 60) as roundedTime'));
        }
        let result = new interaction_model_1.InteractionView;
        builderTotal
            .then((res) => {
            result.totalRecords = res[0].totalRecords;
            result.totalTime = res[0].totalTime === null ? 0 : res[0].totalTime;
            result.roundedTime = res[0].roundedTime === null ? 0 : res[0].roundedTime;
            builderResult.clearSelect();
            builderResult.distinct(callDB.raw(selectResultRaw));
            if (limit != 0)
                builderResult.limit(limit);
            builderResult.orderBy(StartDateColumn);
            return new Promise((resolve, reject) => {
                return resolve(builderResult);
            });
        })
            .then((res) => {
            result.interactions = res;
            return resolve(result);
        })
            .catch(err => {
            logger.info('[HistoricalData::getInteractionsRefactor] Catch error: ' + err);
            reject(err);
        });
    });
};
exports.getInteractionsRefactor = getInteractionsRefactor;
