"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BillingAccountImplementation = void 0;
const log4js = require("log4js");
const promises_shared_1 = require("../shared/promises.shared");
const md5 = require("md5");
const platform_service_1 = require("../services/platform.service");
let icModule = require('intermediate-component-sa');
let logger = log4js.getLogger('ServerScripts');
class BillingAccountImplementation {
    GetSystemInformation(guid) {
        let systemInformationData = {
            installationId: guid,
            macAddress: '',
            publicIp: ''
        };
        return promises_shared_1.DummyPromise()
            .then((result) => {
            if (result === null)
                throw 'SERVER_ERROR_NOT_FOUND_NETWORK_INTERFACE_INFORMATION';
            if (result && result.length) {
                logger.info('[BillingAccountImplementation::GetSystemInformation] guid:', guid, 'getting result[0].mac:', result[0].mac);
                let myNetworkInterface = null;
                myNetworkInterface = result.find(r => r.operstate === 'up')[0].mac;
                systemInformationData.macAddress = myNetworkInterface.mac;
                systemInformationData.publicIp = myNetworkInterface.ip4;
            }
            logger.info('[BillingAccountImplementation::GetSystemInformation] guid:', guid, 'getting systemInformationData:', systemInformationData);
            return Promise.resolve(systemInformationData);
        }).catch((err) => {
            logger.error('[BillingAccountImplementation::GetSystemInformation] guid:', guid, 'getting error:', err);
            return Promise.resolve(false);
        });
    }
    GenerateMD5Content(data) {
        let result = '';
        if (data) {
            result += data.installationId;
            result += data.macAddress;
            logger.info('[BillingAccountImplementation::GenerateMD5Content] getting result:', result);
            result = md5(result);
        }
        return result;
    }
    GetAccountBalance(account) {
        let systemInformationData = null;
        let installationId;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            logger.info('[BillingAccountImplementation::GetPlatformAvailableLicenses] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then((result) => {
            systemInformationData = result;
            let plaformHash = '';
            if (systemInformationData) {
                plaformHash = this.GenerateMD5Content(systemInformationData);
            }
            let body = {
                platformHash: plaformHash,
                installationId: systemInformationData.installationId,
                account: account
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_ACCOUNT_GET_BALANCE', body);
        }).then(result => {
            logger.info('[BillingAccountImplementation::GetPlatformAvailableLicenses] Getting result:', result, 'from external server');
            return Promise.resolve(result);
        }).catch((err) => {
            logger.error('[BillingAccountImplementation::GetPlatformAvailableLicenses] Getting error:', err);
            return Promise.resolve(null);
        });
    }
    GetAccountStatusByVCC(vcc) {
        let installationId;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        })
            .then((result) => {
            logger.info('[BillingAccountImplementation::GetAccountStatusByVCC] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                let body = {
                    installationId: installationId,
                    vcc: vcc
                };
                return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_ACCOUNT_GET_ACCOUNT_STATUS_BY_VCC', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(result => {
            logger.info('[BillingAccountImplementation::GetAccountStatusByVCC] Getting result:', result, 'from external server');
            return Promise.resolve(result);
        })
            .catch((err) => {
            logger.error('[BillingAccountImplementation::GetAccountStatusByVCC] Getting error:', err);
            return Promise.resolve(err);
        });
    }
    GetVCCsByAccount(accountId, type) {
        let systemInformationData = null;
        let installationId;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            logger.info('[BillingAccountImplementation::GetVCCsByAccount] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then((result) => {
            systemInformationData = result;
            let plaformHash = '';
            if (systemInformationData) {
                plaformHash = this.GenerateMD5Content(systemInformationData);
            }
            let body = {
                platformHash: plaformHash,
                installationId: systemInformationData.installationId,
                account: accountId,
                type: type
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_ACCOUNT_GET_VCC', body);
        }).then(result => {
            logger.info('[BillingAccountImplementation::GetVCCsByAccount] Getting result:', result, 'from external server');
            return Promise.resolve(result);
        }).catch((err) => {
            logger.error('[BillingAccountImplementation::GetVCCsByAccount] Getting error:', err);
            return Promise.resolve(null);
        });
    }
    GetMovements(billingMovement) {
        logger.info('[BillingAccountImplementation::GetMovements] Getting billingMovement input:', billingMovement);
        let systemInformationData = null;
        let installationId;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            logger.info('[BillingAccountImplementation::GetMovements] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then((result) => {
            systemInformationData = result;
            let plaformHash = '';
            if (systemInformationData) {
                plaformHash = this.GenerateMD5Content(systemInformationData);
            }
            let body = {
                platformHash: plaformHash,
                installationId: systemInformationData.installationId,
                startDate: billingMovement.startDate,
                endDate: billingMovement.endDate,
                accountId: billingMovement.accountId,
                subAccountId: billingMovement.subAccountId,
                vcc: billingMovement.vcc,
                type: billingMovement.type
            };
            logger.info('[BillingAccountImplementation::GetMovements] Getting body:', body, 'before request api/account/get_movements');
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_ACCOUNT_GET_MOVEMENTS', body);
        }).then(result => {
            logger.info('[BillingAccountImplementation::GetMovements] Getting result:', result, 'from external server');
            let movements = null;
            if (result && result.status) {
                movements = result.data;
            }
            else {
                throw 'ERROR_SERVER_GETTING_MOVEMENTS_DATA';
            }
            return Promise.resolve(movements);
        }).catch((err) => {
            logger.error('[BillingAccountImplementation::GetMovements] Getting error:', err, 'billingMovement input:', billingMovement);
            return Promise.resolve(null);
        });
    }
    GetMovementCurrencies(billingMovement) {
        logger.info('[BillingAccountImplementation::GetMovementCurrencies] Getting billingMovement input:', billingMovement);
        let systemInformationData = null;
        let installationId;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        })
            .then((result) => {
            logger.info('[BillingAccountImplementation::GetMovementCurrencies] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then((result) => {
            systemInformationData = result;
            let plaformHash = '';
            if (systemInformationData) {
                plaformHash = this.GenerateMD5Content(systemInformationData);
            }
            let body = {
                platformHash: plaformHash,
                installationId: systemInformationData.installationId,
                startDate: billingMovement.startDate,
                endDate: billingMovement.endDate,
                accountId: billingMovement.accountId,
                subAccountId: billingMovement.subAccountId,
                vcc: billingMovement.vcc,
                type: billingMovement.type
            };
            logger.info('[BillingAccountImplementation::GetMovementCurrencies] Getting body:', body, 'before request api/account/GetMovementCurrencies');
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_ACCOUNT_GET_MOVEMENT_CURRENCIES', body);
        })
            .then(result => {
            logger.info('[BillingAccountImplementation::GetMovementCurrencies] Getting result:', result, 'from external server');
            let movements = null;
            if (result && result.status) {
                movements = result.data;
            }
            else {
                throw 'ERROR_SERVER_GETTING_MOVEMENT_CURRENCIES_DATA';
            }
            return Promise.resolve(movements);
        })
            .catch((err) => {
            logger.error('[BillingAccountImplementation::GetMovementCurrencies] Getting error:', err, 'billingMovement input:', billingMovement);
            return Promise.resolve(null);
        });
    }
    GetCosts(billingMovement) {
        logger.info('[BillingAccountImplementation::GetCosts] Getting billingMovement input:', billingMovement);
        let systemInformationData = null;
        let installationId;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return platform_service_1.GetPlatformVersion();
        }).then((result) => {
            logger.info('[BillingAccountImplementation::GetCosts] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then((result) => {
            systemInformationData = result;
            let plaformHash = '';
            if (systemInformationData) {
                plaformHash = this.GenerateMD5Content(systemInformationData);
            }
            let body = {
                platformHash: plaformHash,
                installationId: systemInformationData.installationId,
                startDate: billingMovement.startDate,
                endDate: billingMovement.endDate,
                accountId: billingMovement.accountId,
                subAccountId: billingMovement.subAccountId,
                vcc: billingMovement.vcc,
                type: billingMovement.type
            };
            logger.info('[BillingAccountImplementation::GetCosts] Getting body:', body, 'before request api/account/get_movements');
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_ACCOUNT_GET_COSTS', body);
        }).then(result => {
            logger.info('[BillingAccountImplementation::GetCosts] Getting result:', result, 'from external server');
            let costs = null;
            if (result && result.status) {
                costs = result.data;
            }
            else {
                throw 'ERROR_SERVER_GETTING_COSTS_DATA';
            }
            return Promise.resolve(costs);
        }).catch((err) => {
            logger.error('[BillingAccountImplementation::GetCosts] Getting error:', err, 'billingMovement input:', billingMovement);
            return Promise.resolve(null);
        });
    }
}
exports.BillingAccountImplementation = BillingAccountImplementation;
