"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CleanerImplementation = void 0;
const log4js = require("log4js");
const config_1 = require("../config/config");
const promises_shared_1 = require("../shared/promises.shared");
const redis_service_1 = require("../db/redis.service");
const mssql_service_1 = require("../db/mssql.service");
const file_manager_1 = require("../shared/file-manager");
const io_model_1 = require("../models/io.model");
const io_request_1 = require("../io/io.request");
const batch_schedule_implementation_1 = require("./batch-schedule.implementation");
const i6_update_implementation_1 = require("./i6-update.implementation");
let logger = log4js.getLogger('Task');
class CleanerImplementation {
    Execute(executionCode) {
        let totalProgress = [];
        let progress = [];
        let errorProgress = [];
        let summaryProgress = [];
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return this.CleanReadyProcessI6Updates(executionCode);
        })
            .then(_ => {
            return this.CleanReadyProcessInteractions(executionCode);
        })
            .then(_ => {
            return this.CleanReadyStudyCognitiveEvaluations(executionCode);
        })
            .then(_ => {
            return this.CleanReadyEvaluationPlanEvaluations(executionCode);
        })
            .then(_ => {
            return this.CleanReadyEvaluationPlanTipifications(executionCode);
        })
            .then(_ => {
            return redis_service_1.Redis.GetTotalProgress();
        }).then((result) => {
            totalProgress = result;
            return redis_service_1.Redis.GetErrorProgress();
        }).then((result) => {
            errorProgress = result;
            return redis_service_1.Redis.GetProgress();
        }).then((result) => {
            progress = result;
            if (totalProgress && totalProgress.length) {
                totalProgress.forEach(total => {
                    let currentProgress = progress.find(item => item.key === total.key) || {};
                    let currentErrorProgress = errorProgress.find(item => item.key === total.key) || {};
                    total.value = total.value ? total.value : 0;
                    currentProgress.value = currentProgress && currentProgress.value ? currentProgress.value : 0;
                    let realProgress = total.value - currentProgress.value;
                    let summaryItem = JSON.parse(JSON.stringify(total));
                    summaryItem['progress'] = realProgress;
                    summaryItem['total'] = total.value;
                    summaryItem['percentage'] = currentProgress.value <= 0 ? 1 : Math.round((realProgress / total.value) * 100000) / 100000;
                    summaryItem['error'] = currentErrorProgress && currentErrorProgress.value ? currentErrorProgress.value : 0;
                    logger.debug('[CleanerImplementation:Execute] executionCode:', executionCode, 'Progress for ', total.key, {
                        RealProgress: realProgress,
                        TotalValue: total.value,
                        CurrentProgress: currentProgress.value,
                        CurrentErrorProgress: currentErrorProgress?.value ? currentErrorProgress.value : 0,
                        Progress: summaryItem['progress'],
                        Percentaje: summaryItem['percentage'],
                        Error: summaryItem['error']
                    });
                    summaryProgress.push(summaryItem);
                });
            }
            return mssql_service_1.MSSql.BulkProgressTemp(summaryProgress);
        })
            .then(_ => {
            let schedulesToStart = summaryProgress.filter(sp => sp.percentage === 1 && [null, 'ability'].includes(sp.type));
            if (schedulesToStart.length) {
                logger.info(`[CleanerImplementation:Execute] executionCode: ${executionCode} searching schedules for ${schedulesToStart.length} process/evaluationPlan`);
                let batchScheduleImplementation = new batch_schedule_implementation_1.BatchScheduleImplementation();
                return batchScheduleImplementation.GetNewScheduledBatchExecutions(executionCode);
            }
            return Promise.resolve(true);
        })
            .then(_ => {
            let promiseArray = [];
            let process = {};
            for (let item of summaryProgress) {
                let batchName = null;
                switch (item.type) {
                    case 'i6update':
                        batchName = 'I6_UPDATE';
                        break;
                    case 'study':
                        batchName = 'STUDY';
                        break;
                    case 'template':
                        batchName = 'TEMPLATE';
                        break;
                    case 'ability':
                        batchName = 'EVALUATION_PLAN';
                        break;
                    default:
                        batchName = 'TRANSCRIPTION';
                        item.typeId = item.processId;
                        break;
                }
                if (item.percentage === 1) {
                    logger.info('[CleanerImplementation:Execute] executionCode:', executionCode, 'FINALIZED', item);
                    promiseArray.push(redis_service_1.Redis.DeleteTotalProgress(item.key));
                    promiseArray.push(redis_service_1.Redis.DeleteProgress(item.key));
                    promiseArray.push(redis_service_1.Redis.DeleteErrorProgress(item.key));
                    if (['study', 'template'].includes(item.type))
                        promiseArray.push(redis_service_1.Redis.RemoveStudyTemplateThemes(item.vcc, item.typeId));
                    if (['ability'].includes(item.type))
                        promiseArray.push(redis_service_1.Redis.RemoveEvaluationPlanAbilities(item.vcc, item.typeId));
                    if (batchName) {
                        promiseArray.push(redis_service_1.Redis.DeleteBatchRasaModels(batchName.toUpperCase() + '::' + item.typeId));
                    }
                    if (item.key.includes(':i6update')
                        &&
                            item.total > 0) {
                        let i6UpdateImplementation = new i6_update_implementation_1.I6UpdateImplementation();
                        promiseArray.push(i6UpdateImplementation.AddProcessQueue(item.vcc, item.processId, item.key));
                    }
                }
                process[`${item.vcc}-${batchName}`] = {
                    vcc: item.vcc, type: batchName
                };
            }
            if (Object.keys(process).length > 0) {
                let events = [];
                for (let p in process) {
                    events.push(new io_model_1.IOEvent(io_model_1.VCC_EVENT, `UPDATE_${process[p].type}`, null, process[p].vcc));
                }
                promiseArray.push(io_request_1.SendIOEvents(events));
            }
            return Promise.all(promiseArray);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation:Execute] executionCode:', executionCode, 'Progress getting error:', errorMessage);
            return Promise.resolve(false);
        }).then(_ => {
            logger.info('[CleanerImplementation:Execute] executionCode: CleanupTempFile', executionCode, 'getting result:', 'DELETED FILES');
            let promiseArray = [];
            promiseArray.push(file_manager_1.CleanupTempFile(config_1.PATH_FOLDER_TEMP_AUDIO, config_1.PERSISTENCE_AUDIO_TEMP));
            return Promise.all(promiseArray);
        }).then((result) => {
            logger.info('[CleanerImplementation:Execute] executionCode:', executionCode, 'getting result:', 'TASK FINISHED');
            return Promise.resolve(true);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation:Execute] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    CleanReadyProcessI6Updates(executionCode) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return redis_service_1.Redis.GetReadyI6UpdateInteractions(executionCode);
        })
            .then((interactions) => {
            logger.info('[CleanerImplementation::CleanReadyProcessI6Updates] executionCode:', executionCode, 'getting: interactions:', (interactions?.length ? interactions.length : 0), 'after Redis.GetReadyProcessI6Update');
            return Promise.resolve(true);
        })
            .then(_ => {
            let hash = 'I6Update:' + executionCode;
            return redis_service_1.Redis.Del(hash, executionCode);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation::CleanReadyProcessI6Updates] executionCode:', executionCode, 'BulkProcessInteractionTemp getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    CleanReadyProcessInteractions(executionCode) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return redis_service_1.Redis.GetAllReadyProcessInteraccionStorages(executionCode);
        })
            .then(storages => {
            logger.debug(`[CleanerImplementation::CleanReadyProcessInteractions] Storages ${storages}`);
            if (storages?.length) {
                let oldStorages = storages.filter(s => !s.includes(executionCode));
                if (oldStorages) {
                    logger.info(`[CleanerImplementation::CleanReadyProcessInteractions] executionCode: ${executionCode}, ${oldStorages} old storages will be taked.`);
                }
                let promiseArray = [];
                for (let i = 0; i < 5; i++) {
                    if (storages[i])
                        promiseArray.push(this.SaveReadyProcessInteractions(storages[i], executionCode));
                }
                return Promise.all(promiseArray);
            }
            return Promise.resolve([true]);
        })
            .then(result => {
            if (result.includes(false))
                logger.warn(`[CleanerImplementation::CleanReadyProcessInteractions] executionCode: ${executionCode}, ${result.filter(r => r === false).length} storages could not be saved.`);
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation::CleanReadyProcessInteractions] executionCode:', executionCode, 'BulkProcessInteractionTemp getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    SaveReadyProcessInteractions(storage, executionCode) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (storage !== executionCode)
                logger.info(`[CleanerImplementation::SaveReadyProcessInteractions] executionCode: ${executionCode}, searching old starage from execution ${storage}`);
            return redis_service_1.Redis.GetReadyProcessInteractions(storage);
        })
            .then((interactions) => {
            logger.info('[CleanerImplementation::SaveReadyProcessInteractions] executionCode:', executionCode, 'getting: interactions:', (interactions && interactions.length ? interactions.length : 0), 'after Redis.GetReadyProcessInteractions');
            if (interactions && interactions.length) {
                return mssql_service_1.MSSql.BulkProcessInteractionTemp(interactions);
            }
            return Promise.resolve(true);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_BULK_PROCESS_INTERACTIONS_FAILED';
            let hash = 'Audios:' + storage;
            return redis_service_1.Redis.Del(hash, executionCode);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation::SaveReadyProcessInteracctions] executionCode:', executionCode, 'BulkProcessInteractionTemp getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    CleanReadyStudyCognitiveEvaluations(executionCode) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return redis_service_1.Redis.GetReadyCognitiveInteractions(executionCode);
        })
            .then((interactions) => {
            logger.info('[CleanerImplementation::CleanReadyStudyCognitiveEvaluations] executionCode:', executionCode, 'getting: interactions:', (interactions && interactions.length ? interactions.length : 0), 'after Redis.GetReadyCognitiveInteractions');
            if (interactions && interactions.length) {
                return mssql_service_1.MSSql.BulkInteractionCognitiveTemp(interactions);
            }
            return Promise.resolve(true);
        })
            .then((result) => {
            let hash = 'Cognitive:' + executionCode;
            return redis_service_1.Redis.Del(hash, executionCode);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation::CleanReadyStudyCognitiveEvaluations] executionCode:', executionCode, 'BulkInteractionCognitiveTemp getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    CleanReadyEvaluationPlanEvaluations(executionCode) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return redis_service_1.Redis.GetAllReadyCognitiveAbilityStorages(executionCode);
        })
            .then(storages => {
            logger.debug(`[CleanerImplementation::CleanReadyEvaluationPlanEvaluations] Storages ${storages}`);
            if (storages?.length) {
                let oldStorages = storages.filter(s => !s.includes(executionCode));
                if (oldStorages)
                    logger.info(`[CleanerImplementation::CleanReadyEvaluationPlanEvaluations] executionCode: ${executionCode}, ${oldStorages} old storages will be taked.`);
            }
            let promiseArray = [];
            for (let i = 0; i < 5; i++) {
                if (storages[i])
                    promiseArray.push(this.SaveReadyEvaluationPlanEvaluations(storages[i], executionCode));
            }
            return Promise.all(promiseArray);
        })
            .then(result => {
            if (result.includes(false))
                logger.warn(`[CleanerImplementation::CleanReadyEvaluationPlanEvaluations] executionCode: ${executionCode}, ${result.filter(r => r === false).length} storages could not be saved.`);
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation::CleanReadyEvaluationPlanEvaluations] executionCode:', executionCode, 'BulkAbilityCognitiveTemp getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    SaveReadyEvaluationPlanEvaluations(storage, executionCode) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (storage !== executionCode)
                logger.info(`[CleanerImplementation::SaveReadyEvaluationPlanEvaluations] executionCode: ${executionCode}, searching old starage from execution ${storage}`);
            return redis_service_1.Redis.GetReadyCognitiveAbilities(storage);
        })
            .then((interactions) => {
            logger.info('[CleanerImplementation::SaveReadyEvaluationPlanEvaluations] executionCode:', executionCode, 'getting: interactions:', (interactions?.length ? interactions.length : 0), 'after Redis.GetReadyTipificationInteractions');
            if (interactions?.length) {
                return mssql_service_1.MSSql.BulkAbilityCognitiveTemp(interactions);
            }
            return Promise.resolve(true);
        })
            .then(_ => {
            let hash = 'AbilityCognitive:' + storage;
            return redis_service_1.Redis.Del(hash, executionCode);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation::SaveReadyEvaluationPlanEvaluations] executionCode:', executionCode, 'BulkAbilityCognitiveTemp getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    CleanReadyEvaluationPlanTipifications(executionCode) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return redis_service_1.Redis.GetAllReadyTipificationInteractionStorages(executionCode);
        })
            .then(storages => {
            logger.debug(`[CleanerImplementation::CleanReadyEvaluationPlanTipifications] Storages ${storages}`);
            let promiseArray = [];
            if (storages?.length) {
                let oldStorages = storages.filter(s => !s.includes(executionCode)).length;
                if (oldStorages)
                    logger.info(`[CleanerImplementation::CleanReadyEvaluationPlanTipifications] executionCode: ${executionCode}, ${oldStorages} old storages will be taked.`);
                for (let i = 0; i < 5; i++) {
                    if (storages[i])
                        promiseArray.push(this.SaveReadyEvaluationPlanTipifications(storages[i], executionCode));
                }
            }
            return Promise.all(promiseArray);
        })
            .then(result => {
            if (result.includes(false))
                logger.warn(`[CleanerImplementation::CleanReadyEvaluationPlanTipifications] executionCode: ${executionCode}, ${result.filter(r => r === false).length} storages could not be saved.`);
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation::CleanReadyEvaluationPlanTipifications] executionCode:', executionCode, 'BulkProcessInteractionTemp getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    SaveReadyEvaluationPlanTipifications(storage, executionCode) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (storage !== executionCode)
                logger.info(`[CleanerImplementation::SaveReadyEvaluationPlanTipifications] executionCode: ${executionCode}, searching old starage from execution ${storage}`);
            return redis_service_1.Redis.GetReadyTipificationInteractions(storage);
        })
            .then((interactions) => {
            logger.info('[CleanerImplementation::SaveReadyEvaluationPlanTipifications] executionCode:', executionCode, 'getting: interactions:', (interactions?.length ? interactions.length : 0), 'after Redis.GetReadyTipificationInteractions');
            if (interactions?.length) {
                return mssql_service_1.MSSql.BulkTipificationInteractionTemp(interactions);
            }
            return Promise.resolve(true);
        })
            .then(_ => {
            let hash = 'ReadyTipificationInteraction:' + storage;
            return redis_service_1.Redis.Del(hash, executionCode);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[CleanerImplementation::SaveReadyEvaluationPlanTipifications] executionCode:', executionCode, 'TipificationInteractionTemp getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
}
exports.CleanerImplementation = CleanerImplementation;
