"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CognitiveEngineMigrateImplementation = void 0;
const redis_service_1 = require("../db/redis.service");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const nlu_1 = require("./cognitive-engine-migration/nlu");
const entity_1 = require("./cognitive-engine-migration/entity");
const enum_1 = require("./cognitive-engine-migration/enum");
const intent_1 = require("./cognitive-engine-migration/intent");
const utterance_1 = require("./cognitive-engine-migration/utterance");
let logger = log4js.getLogger('Task');
class CognitiveEngineMigrateImplementation {
    Execute(executionCode) {
        logger.info('[CognitiveEngineMigrateImplementation::Execute] executionCode:', executionCode);
        return promises_shared_1.SleepPromise(5).then(_ => {
            return redis_service_1.Redis.GetCognitiveEngineMigrationData();
        })
            .then(results => {
            if (typeof results != "object" || !results) {
                logger.debug("[CognitiveEngineMigrateImplementation::Execute] NOT_FOUND_DATA ", results);
                return [false];
            }
            logger.info('[CognitiveEngineMigrateImplementation::Execute] FOUND_DATA_TO_MIGRATE: ', results);
            let promises = [];
            for (let vcc in results) {
                let data = JSON.parse(results[vcc]);
                switch (data.state) {
                    case 0:
                        promises.push(this.StartMigration(vcc, data));
                        break;
                    case 1:
                        let migrator = new nlu_1.NluMigrator();
                        promises.push(migrator.MigrateNlus(vcc, data, 0, enum_1.ITEMS_PER_PAGE));
                        break;
                    default: break;
                }
            }
            return Promise.all(promises);
        })
            .then((results) => {
            logger.info('[CognitiveEngineMigrateImplementation::Execute] Before task promises results ', results);
            return redis_service_1.Redis.GetCognitiveEngineMigrationData();
        })
            .then(tasks => {
            if (typeof tasks != "object" || !tasks) {
                return [false];
            }
            let taskPromises = [];
            for (let vcc in tasks) {
                let data = JSON.parse(tasks[vcc]);
                taskPromises.push(this.CheckStatus(vcc, data));
            }
            return Promise.all(taskPromises);
        })
            .then((results) => {
            let hasSuccess = !results.includes(false);
            logger.info('[CognitiveEngineMigrateImplementation::Execute] Final result ', hasSuccess);
            return hasSuccess;
        })
            .catch(err => {
            logger.error('[CognitiveEngineMigrateImplementation::Execute] Error ocurred on migrate', err);
            return false;
        });
    }
    CheckStatus(vcc, data) {
        let promises = [];
        let totalMigration = false;
        logger.info("[CognitiveEngineMigrateImplementation::CheckStatus] Checking status migration for ", vcc, data);
        let hasCreatedNlu = (data.totalCreatedNlu + data.totalErrorNlu) >= data.totalNlu;
        if (hasCreatedNlu) {
            promises.push(mssql_service_1.MSSql.UpdateVccStatus(vcc, 0));
        }
        if (data.totalNlu * 2 == data.totalDeletedNlu + data.totalCreatedNlu) {
            logger.info("[CognitiveEngineMigrateImplementation::CheckStatus] Deleting keys for nlu");
            promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey(`${enum_1.ROOT_KEY}:${vcc}:Nlus`));
            promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey(`${enum_1.ROOT_KEY}:${vcc}:Nlus:Keys`));
            if (data.newEngine === 'NEW_RASA_ENGINE')
                totalMigration = true;
        }
        if (data.totalIntent > 0 && hasCreatedNlu) {
            let im = new intent_1.IntentMigrator();
            promises.push(im.MigrateIntents(vcc, data, 0, enum_1.ITEMS_PER_PAGE));
            if (data.totalIntent == data.totalCreatedIntent) {
                logger.info("[CognitiveEngineMigrateImplementation::CheckStatus] Deleting keys for intent");
                promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey(`${enum_1.ROOT_KEY}:${vcc}:Intents`));
                promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey(`${enum_1.ROOT_KEY}:${vcc}:Intents:Keys`));
            }
            if (data.totalUtterance > 0 && (data.totalCreatedIntent + data.totalErrorIntent) == data.totalIntent) {
                let um = new utterance_1.UtteranceMigrator();
                promises.push(um.MigrateUtterances(vcc, data, 0, enum_1.ITEMS_PER_PAGE));
                if (data.totalUtterance == data.totalCreatedUtterance) {
                    logger.info("[CognitiveEngineMigrateImplementation::CheckStatus] Deleting keys for utterance");
                    promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey(`${enum_1.ROOT_KEY}:${vcc}:Utterances`));
                    promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey(`${enum_1.ROOT_KEY}:${vcc}:Utterances:Keys`));
                }
            }
        }
        if (data.totalEntity > 0 && hasCreatedNlu) {
            let em = new entity_1.EntityMigrator();
            promises.push(em.MigrateEntities(vcc, data, 0, enum_1.ITEMS_PER_PAGE));
            if (data.totalEntity == data.totalCreatedEntity) {
                logger.info("[CognitiveEngineMigrateImplementation::CheckStatus] Deleting keys for entity");
                promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey(`${enum_1.ROOT_KEY}:${vcc}:Entities`));
                promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey(`${enum_1.ROOT_KEY}:${vcc}:Entities:Keys`));
            }
        }
        let finished = false;
        if ((data.totalNlu * 2 + data.totalUtterance + data.totalEntity + data.totalIntent) ==
            (data.totalDeletedNlu + data.totalCreatedNlu +
                data.totalCreatedUtterance +
                data.totalCreatedIntent +
                data.totalCreatedEntity)) {
            finished = true;
            logger.info("[CognitiveEngineMigrateImplementation::CheckStatus] Deleting all keys for ", vcc);
            promises.push(this.DeleteMigrationTask(vcc));
            promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey(`${enum_1.ROOT_KEY}:${vcc}:AppIds`));
            promises.push(redis_service_1.Redis.DeleteCognitiveEngineMigrationKey(`${enum_1.ROOT_KEY}:${vcc}:IntentIds`));
        }
        promises.push(redis_service_1.Redis.UpdateCognitiveEngineMigrationData(vcc, Object.assign(data, {
            counter: data.counter + 1
        })));
        if (finished || data.counter > 10) {
            if (totalMigration)
                promises.push(mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'nlu', 'AVAILABLE_TM', 1, enum_1.AVAILABLE));
            promises.push(mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'nlu', enum_1.AVAILABLE, 1, enum_1.AVAILABLE));
        }
        return Promise.all(promises).then((results) => {
            return !results.includes(false);
        });
    }
    DeleteMigrationTask(vcc) {
        logger.info('[CognitiveEngineMigrateImplementation::DeleteMigrationTask] Deleting task for ', vcc);
        return redis_service_1.Redis.DeleteCognitiveEngineMigrationTask(enum_1.CONFIG_KEY, vcc).catch((err) => {
            logger.error('[CognitiveEngineMigrateImplementation::DeleteMigrationTask] Error on delete task for redis:', vcc, err);
            return Promise.resolve(false);
        });
    }
    StartMigration(vcc, data) {
        let listMode = 1;
        let totalMigration = false;
        if (data.oldEngine === 'OLD_RASA_ENGINE') {
            listMode = 3;
            totalMigration = true;
        }
        let nlusToMigrate = '';
        return mssql_service_1.MSSql.GetNluIntentUtteranceToExport(vcc, listMode).then(result => {
            let promises = [];
            if (result.nlus.length > 0) {
                nlusToMigrate = result.nlus.map(nlu => nlu.appId).join('\',\'');
                promises.push(this.SaveDataToRedis(vcc, result.nlus, 'Nlus', 'appId'));
            }
            if (result.intents.length > 0) {
                promises.push(this.SaveDataToRedis(vcc, result.intents, 'Intents', 'key'));
                if (result.utterances.length > 0) {
                    promises.push(this.SaveDataToRedis(vcc, result.utterances, 'Utterances', 'key'));
                }
            }
            if (result.entities.length > 0) {
                promises.push(this.SaveDataToRedis(vcc, result.entities, 'Entities', 'key'));
            }
            data.totalNlu = result.nlus.length;
            data.totalIntent = result.intents.length;
            data.totalUtterance = result.utterances.length;
            data.totalEntity = result.entities.length;
            return Promise.all(promises);
        }).then((results) => {
            let errors = results.filter(r => r.includes("ERROR"));
            if (errors.length > 0) {
                logger.error('[CognitiveEngineMigrateImplementation::StartMigration] AFTER SAVE DATA TO REDIS:', errors);
                return Promise.all([
                    false,
                    mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, '*', '*', 3, enum_1.CANNOT_SAVE_DATA_TO_REDIS)
                ]);
            }
            return Promise.all([
                true,
                redis_service_1.Redis.UpdateCognitiveEngineMigrationData(vcc, Object.assign(data, { state: 1 })),
                totalMigration ? mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'nlu', 'TOTAL_MIGRATION', 2, enum_1.MIGRATING, nlusToMigrate) : mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, '*', '*', 2, enum_1.MIGRATING)
            ]);
        }).then((status) => {
            return status[0];
        }).catch(err => {
            logger.error('[CognitiveEngineMigrateImplementation::StartMigration] Error ocurred on start migration:', err);
            return mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, '*', '*', 3, enum_1.WITH_ERROR);
        });
    }
    SaveDataToRedis(vcc, rows, key, id) {
        return redis_service_1.Redis.SaveCognitiveEngineMigrationModel(`${enum_1.ROOT_KEY}:${vcc}:${key}`, id, rows).then((saved) => {
            return saved ? `SUCCESS_SAVE_${key}_TO_REDIS` : `ERROR_ON_SAVE_${key}_TO_REDIS`;
        });
    }
}
exports.CognitiveEngineMigrateImplementation = CognitiveEngineMigrateImplementation;
