"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProcessImplementation = void 0;
const redis_service_1 = require("../db/redis.service");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const config_1 = require("../config/config");
const evaluation_plan_model_1 = require("../models/evaluation-plan.model");
const billing_implementation_1 = require("../implementation/billing.implementation");
const template_service_1 = require("../services/template.service");
const arrays_shared_1 = require("../shared/arrays.shared");
let logger = log4js.getLogger('Task');
class ProcessImplementation {
    Execute(executionCode, rateRecursive) {
        let executeIndex = 1;
        let ratio = config_1.RATIO_PROCESS;
        let executeResult = true;
        return redis_service_1.Redis.LRange('Process', 0, ratio - 1)
            .then((result) => {
            let promiseArray = [];
            let filteredList = arrays_shared_1.GetUniqueFromArray(result);
            ratio = filteredList?.length <= ratio ? filteredList.length : 0;
            logger.info('[ProcessImplementation::Execute] executionCode:', executionCode, 'ratio:', ratio, 'getting Redis.LRange:', filteredList);
            while (ratio >= executeIndex) {
                promiseArray.push(this.ProcessExecute(executionCode));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then((results) => {
            if (results.filter(item => item === false).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('Process', 0, ratio - 1);
        }).then((result) => {
            if (result && result.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1);
            return executeResult;
        }).then(result => {
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[ProcessImplementation::Execute] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    ProcessExecute(executionCode) {
        let currentQueueId = '';
        let vcc = '';
        let processId = '';
        let type = '';
        let typeId = '';
        return redis_service_1.Redis.RPopLPush('Process', 'ProcessExecute')
            .then((queueId) => {
            if (queueId?.length > 0) {
                currentQueueId = queueId;
                logger.info('[ProcessImplementation::ProcessExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId);
                let item = queueId.split(':');
                if (item && item.length) {
                    vcc = item[0];
                    processId = item[1];
                    type = item[2] ? item[2] : null;
                    typeId = item[3] ? item[3] : null;
                    let promiseArray = [];
                    logger.debug('[ProcessImplementation::ProcessExecute] Creating progress keys for process ' + processId);
                    promiseArray.push(redis_service_1.Redis.SaveTotalProgress(currentQueueId, 0));
                    promiseArray.push(redis_service_1.Redis.SaveProgress(currentQueueId, 0));
                    promiseArray.push(redis_service_1.Redis.SaveErrorProgress(currentQueueId, 0));
                    return Promise.all(promiseArray);
                }
                else {
                    logger.error('[ProcessImplementation::ProcessExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return Promise.resolve(null);
        }).then((_) => {
            if (vcc && processId && ['study', 'template'].includes(type) && typeId) {
                return this.ProcessStudy(vcc, processId, type, typeId, executionCode, currentQueueId);
            }
            else if (vcc && processId && type === 'evaluationPlan' && typeId) {
                return this.ProcessEvaluationPlan(vcc, processId, typeId, executionCode, currentQueueId);
            }
            else if (vcc && processId && type === 'ability' && typeId) {
                return this.ProcessAbility(vcc, processId, typeId, executionCode, currentQueueId);
            }
            else if (vcc && processId && type === 'tipification' && typeId) {
                return this.ProcessEvaluationPlanTipification(vcc, processId, typeId, executionCode, currentQueueId);
            }
            else if (vcc && processId && type === 'i6update') {
                return this.ProcessI6Update(vcc, processId, executionCode, currentQueueId);
            }
            else if (vcc && processId && type === null) {
                return this.ProcessAudio(vcc, processId, executionCode, currentQueueId);
            }
            return Promise.resolve(null);
        }).then((result) => {
            if (result) {
                return redis_service_1.Redis.DeleteQueue('ProcessExecute', currentQueueId);
            }
            else {
                logger.warn('[ProcessImplementation::ProcessExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVE');
                return Promise.resolve(false);
            }
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[ProcessImplementation::ProcessExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    ProcessI6Update(vcc, processId, executionCode, currentQueueId) {
        let interactionsToUpdate = [];
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetProcessI6InteractionsToUpdateTask(vcc, processId);
        })
            .then(result => {
            if (!result.status)
                throw 'SERVER_ERROR_GET_I6_INTERACTIONS_TASK_FAILED';
            interactionsToUpdate = result?.data?.length ? result.data : [];
            if (interactionsToUpdate?.length) {
                let promiseArray = [];
                logger.info('[ProcessImplementation::ProcessI6Update] Getting i6 call interactions to update: ' + interactionsToUpdate.length);
                interactionsToUpdate.forEach((itu) => {
                    promiseArray.push(this.ProcessI6UpdateExecute(itu, currentQueueId, executionCode));
                });
                return Promise.all(promiseArray);
            }
            else {
                if (currentQueueId)
                    logger.warn('[ProcessImplementation::ProcessI6Update] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'GetProcessI6InteractionsToUpdateTask: 0');
                let processQueue = `${vcc}:${processId}`;
                logger.error('[ProcessImplementation::ProcessI6Update] Creating process queue');
                return redis_service_1.Redis.CreateQueueL('ProcessExecute', processQueue);
            }
        })
            .then(_ => {
            let promiseArray = [];
            logger.debug('[ProcessImplementation::ProcessI6Update] Creating progress keys for process ' + processId);
            promiseArray.push(redis_service_1.Redis.SaveTotalProgress(currentQueueId, interactionsToUpdate.length));
            promiseArray.push(redis_service_1.Redis.SaveProgress(currentQueueId, interactionsToUpdate.length));
            return Promise.all(promiseArray);
        })
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[ProcessImplementation::ProcessI6Update] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    ProcessAudio(vcc, processId, executionCode, currentQueueId) {
        let totalAllAudio = 0;
        let totalNewAudio = 0;
        let totalOldAudio = 0;
        let totalCancelledInteraction = 0;
        let batchTypeName = '';
        let batchTypeId = '';
        let processInteractions = [];
        let processEnvironment = {};
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetProcessInteractionsTask(vcc, processId);
        })
            .then((result) => {
            if (!result.status)
                throw 'SERVER_ERROR_GET_INTERACTIONS_TASK_FAILED';
            processInteractions = result && result.data && result.data.length ? result.data : [];
            if (!processInteractions.length)
                throw 'SERVER_ERROR_NO_INTERACTIONS_FOR_PROCESS';
            processEnvironment.vcc = processInteractions[0].vcc;
            processEnvironment.name = processInteractions[0].name;
            processEnvironment.type = processInteractions[0].environmentType;
            return mssql_service_1.MSSql.GetEvaluationPlanByProcessId(vcc, processId);
        })
            .then(result => {
            if (result !== null) {
                batchTypeName = 'EVALUATION_PLAN';
                batchTypeId = result.id;
            }
            else {
                batchTypeName = 'TRANSCRIPTION';
                batchTypeId = processId;
            }
            if (processInteractions?.length) {
                totalNewAudio = processInteractions.length;
                totalCancelledInteraction = processInteractions.filter(pi => pi.isCancel).length;
                logger.info('[ProcessImplementation::ProcessAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'GetProcessInteractionsTask:', processInteractions.length);
                let promiseArray = [];
                let billing = new billing_implementation_1.BillingImplementation();
                logger.info('[ProcessImplementation::ProcessAudio] Adding new Transcription to MainFile: ' + processInteractions.length);
                processInteractions.forEach((processInteraction) => {
                    processInteraction.batchTypeName = batchTypeName;
                    processInteraction.batchTypeId = batchTypeId;
                    if (processInteraction.needI6Update && processInteraction.errorMessage) {
                        processInteraction.status = -1;
                        processInteraction.statusDescription = 'pendingToRemove';
                        processInteraction.isCancel = true;
                        processInteraction.retry = 100;
                    }
                    promiseArray.push(this.ProcessInteractionExecute(processInteraction, currentQueueId, executionCode));
                    billing.AddInsertTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId);
                });
                return Promise.all(promiseArray);
            }
            else {
                if (currentQueueId)
                    logger.warn('[ProcessImplementation::ProcessAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'GetProcessInteractionsTask: 0');
                return Promise.resolve(null);
            }
        })
            .then((result) => {
            if (result && result.length) {
                let errors = result.filter(item => item === false);
                if (errors && errors.length) {
                    logger.error('[ProcessImplementation::ProcessAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'errors found after ProcessInteractionExecute total:', errors.length);
                    return Promise.resolve(false);
                }
            }
            return Promise.resolve(true);
        })
            .then(_ => {
            let promiseArray = [];
            logger.debug('[ProcessImplementation::ProcessAudio] Creating progress keys for process ' + processId);
            promiseArray.push(redis_service_1.Redis.SaveTotalProgress(currentQueueId, totalNewAudio));
            promiseArray.push(redis_service_1.Redis.SaveProgress(currentQueueId, totalNewAudio));
            promiseArray.push(redis_service_1.Redis.SaveProcessEnvironment(`${vcc}:${processId}`, processEnvironment));
            return Promise.all(promiseArray);
        })
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[ProcessImplementation::ProcessAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            if (errorMessage === 'SERVER_ERROR_NO_INTERACTIONS_FOR_PROCESS')
                return Promise.resolve(true);
            else
                return Promise.resolve(false);
        });
    }
    ProcessStudy(vcc, processId, batchType, studyId, executionCode, currentQueueId) {
        let totalStudy = 0;
        let newQueueId = '';
        let studyNluModels = [];
        let studyNluRasaModels = [];
        let billing = new billing_implementation_1.BillingImplementation();
        let processStudy = null;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetProcessTask(vcc, processId);
        })
            .then((result) => {
            if (result && result.status === 'FAILED')
                throw 'CANCEL_PROCESS_FAILED';
            if (result && result.status === 'COMPLETE')
                return Promise.resolve(true);
            throw 'CANCEL_PROCESS_PENDING';
        })
            .then(result => {
            return mssql_service_1.MSSql.GetStudy(vcc, studyId);
        })
            .then(result => {
            processStudy = result;
            return this.GetStudyTemplateThemes(vcc, processStudy.templateId);
        })
            .then(result => {
            return redis_service_1.Redis.SaveStudyTemplateThemes(vcc, studyId, result);
        })
            .then(result => {
            return mssql_service_1.MSSql.GetStudyGenericQueryTask(vcc, processId, studyId);
        })
            .then((result) => {
            let promiseArray = [];
            if (result && result.length) {
                totalStudy = result.length;
                logger.info('[ProcessImplementation::ProcessStudy] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetStudyGenericQueryTask:', result.length);
                result.forEach((studyQuery) => {
                    newQueueId = studyQuery.vcc;
                    newQueueId += ':';
                    newQueueId += studyQuery.processId;
                    newQueueId += ':';
                    newQueueId += studyQuery.queryId;
                    newQueueId += ':';
                    newQueueId += batchType;
                    newQueueId += ':';
                    newQueueId += studyQuery.studyId;
                    promiseArray.push(redis_service_1.Redis.CreateQueueL('SemanticQuery', newQueueId));
                    billing.AddInsertSemanticAnalysisDetailToMainFile(studyQuery.vcc, batchType.toUpperCase(), studyQuery.studyId, studyQuery.processId, studyQuery.queryId);
                });
            }
            else {
                if (currentQueueId)
                    logger.warn('[ProcessImplementation::ProcessStudy] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetStudyGenericQueryTask: 0');
            }
            return Promise.all(promiseArray);
        })
            .then((result) => {
            if (result && result.length) {
                let errors = result.filter(item => item === false);
                if (errors && errors.length) {
                    logger.error('[ProcessImplementation::ProcessStudy] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'errors found after CreateQueueL -> SemanticQuery total:', errors.length);
                    return Promise.resolve(false);
                }
            }
            return Promise.resolve(true);
        })
            .then(result => {
            return mssql_service_1.MSSql.GetStudyCognitiveInteractionsTask(vcc, processId, studyId);
        })
            .then((result) => {
            if (!result.status)
                throw 'SERVER_ERROR_GET_INTERACTIONS_TASK_FAILED';
            let processInteractions = result && result.data && result.data.length ? result.data : [];
            if (processInteractions && processInteractions.length) {
                totalStudy += processInteractions.length;
                logger.info('[ProcessImplementation::ProcessStudy] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetStudyCognitiveInteractionsTask:', processInteractions.length);
                let promiseArray = [];
                processInteractions.forEach((interaction, i) => {
                    promiseArray.push(this.CognitiveInteractionExecute(batchType, studyId, interaction, currentQueueId, executionCode));
                    if (studyNluModels.filter(n => n === interaction.nluId).length === 0)
                        studyNluModels.push(interaction.nluId);
                    if (studyNluRasaModels.filter(n => n === interaction.nluId).length === 0 && interaction.cognitiveEngine && interaction.cognitiveEngine.includes('RASA'))
                        studyNluRasaModels.push(interaction.nluId);
                });
                studyNluModels.map(n => {
                    billing.AddInsertCognitiveAnalysisDetailToMainFile(vcc, batchType.toUpperCase(), studyId, processId, n);
                });
                if (!promiseArray.length)
                    promiseArray.push(Promise.resolve(true));
                return Promise.all(promiseArray);
            }
            else {
                if (currentQueueId)
                    logger.warn('[ProcessImplementation::ProcessStudy] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting InteractionCognitive: 0');
                return Promise.resolve(null);
            }
        })
            .then((result) => {
            if (result && result.length) {
                let errors = result.filter(item => item === false);
                if (errors && errors.length) {
                    logger.error('[ProcessImplementation::ProcessStudy] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'errors found after CreateQueueL -> CognitiveAnalysis total:', errors.length);
                    return Promise.resolve(false);
                }
            }
            return Promise.resolve(true);
        })
            .then(result => {
            if (studyNluModels.length > 0)
                return this.RunCurrentBatchesRasaModels(vcc, batchType.toUpperCase(), studyId, studyNluRasaModels, executionCode, currentQueueId);
            else
                return Promise.resolve(true);
        })
            .then(result => {
            let promiseArray = [];
            logger.debug('[ProcessImplementation::ProcessStudy] Creating progress keys for process ' + processId);
            promiseArray.push(redis_service_1.Redis.SaveTotalProgress(currentQueueId, totalStudy));
            promiseArray.push(redis_service_1.Redis.SaveProgress(currentQueueId, totalStudy));
            return Promise.all(promiseArray);
        })
            .then(result => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            if (errorMessage === 'CANCEL_PROCESS_FAILED') {
                logger.warn('[ProcessImplementation::ProcessStudy] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                return Promise.resolve(true);
            }
            else if (errorMessage === 'CANCEL_PROCESS_PENDING') {
                logger.warn('[ProcessImplementation::ProcessStudy] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                return Promise.resolve(false);
            }
            logger.error('[ProcessImplementation::ProcessStudy] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    ProcessEvaluationPlan(vcc, processId, evaluationPlanId, executionCode, currentQueueId) {
        let newQueueId = '';
        let isError = false;
        return promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetProcessTask(vcc, processId);
        }).then((result) => {
            if (result?.status.includes('FAILED'))
                throw 'CANCEL_PROCESS_FAILED';
            if (result && result.status === 'COMPLETE')
                return Promise.resolve(true);
            throw 'CANCEL_PROCESS_PENDING';
        }).then((result) => {
            if (result) {
                return mssql_service_1.MSSql.GetEvaluationPlanTask(vcc, evaluationPlanId);
            }
            return null;
        }).then((result) => {
            if (result) {
                newQueueId = vcc;
                newQueueId += ':';
                newQueueId += processId;
                newQueueId += ':';
                newQueueId += result.filterByContentQueryName;
                newQueueId += ':';
                newQueueId += 'evaluationPlan';
                newQueueId += ':';
                newQueueId += evaluationPlanId;
                return redis_service_1.Redis.CreateQueueL('SemanticQuery', newQueueId);
            }
            return null;
        }).then(result => {
            let promiseArray = [];
            logger.debug('[ProcessImplementation::ProcessEvaluationPlan] Creating progress keys for process ' + processId);
            promiseArray.push(redis_service_1.Redis.SaveRetry(newQueueId, 0));
            promiseArray.push(redis_service_1.Redis.SaveTotalProgress(currentQueueId, 1));
            promiseArray.push(redis_service_1.Redis.SaveProgress(currentQueueId, 1));
            return Promise.all(promiseArray);
        }).then(result => {
            return true;
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            let promiseArray = [];
            if (errorMessage === 'CANCEL_PROCESS_PENDING') {
                isError = true;
                logger.warn('[ProcessImplementation::ProcessEvaluationPlan] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                promiseArray.push(redis_service_1.Redis.DeleteTotalProgress(currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteProgress(currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteErrorProgress(currentQueueId));
            }
            else {
                throw errorMessage;
            }
            return Promise.all(promiseArray);
        }).then(result => {
            if (isError)
                return Promise.resolve(false);
            return Promise.resolve(true);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            if (errorMessage === 'CANCEL_PROCESS_FAILED') {
                logger.warn('[ProcessImplementation::ProcessEvaluationPlan] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                return Promise.resolve(true);
            }
            else if (errorMessage === 'CANCEL_PROCESS_PENDING') {
                logger.warn('[ProcessImplementation::ProcessEvaluationPlan] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                return Promise.resolve(false);
            }
            logger.error('[ProcessImplementation::ProcessEvaluationPlan] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    ProcessAbility(vcc, processId, evaluationPlanId, executionCode, currentQueueId) {
        let newQueueId = '';
        let processEvaluationPlan = null;
        let totalAbility = 0;
        let isError = false;
        let evaluationPlanNluModels = [];
        let evaluationPlanNluRasaModels = [];
        let billing = new billing_implementation_1.BillingImplementation();
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetProcessEvaluationPlanTask(vcc, processId, evaluationPlanId);
        }).then((result) => {
            if (result?.status === 'FAILED') {
                logger.error('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Process failed, canceling evaluation phase');
                throw 'CANCEL_PROCESS_FAILED';
            }
            processEvaluationPlan = result;
            if (processEvaluationPlan?.status === 'COMPLETE'
                &&
                    processEvaluationPlan?.evaluationPlanStatus === 5
                &&
                    ['A', 'X'].includes(processEvaluationPlan?.evaluationPlanType))
                return Promise.resolve(true);
            throw 'CANCEL_PROCESS_PENDING';
        })
            .then(result => {
            return this.GetEvaluationPlanAbilities(vcc, processEvaluationPlan.evaluationPlanId);
        })
            .then(result => {
            return redis_service_1.Redis.SaveEvaluationPlanAbilities(vcc, processEvaluationPlan.evaluationPlanId, result);
        })
            .then((result) => {
            if (result)
                return mssql_service_1.MSSql.GetEvaluationPlanAbilityGenericQueryTask(vcc, evaluationPlanId);
            return Promise.resolve(null);
        }).then((result) => {
            let promiseArray = [];
            if (result?.length) {
                totalAbility = result.length;
                logger.info('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetEvaluationPlanAbilityGenericQueryTask:', result.length);
                result.forEach((query) => {
                    newQueueId = vcc;
                    newQueueId += ':';
                    newQueueId += processId;
                    newQueueId += ':';
                    newQueueId += query.name;
                    newQueueId += ':';
                    newQueueId += 'ability';
                    newQueueId += ':';
                    newQueueId += evaluationPlanId;
                    promiseArray.push(redis_service_1.Redis.CreateQueueL('SemanticQuery', newQueueId));
                    promiseArray.push(redis_service_1.Redis.SaveRetry(newQueueId, 0));
                    billing.AddInsertSemanticAnalysisDetailToMainFile(vcc, 'EVALUATION_PLAN', evaluationPlanId, processId, query.name);
                });
            }
            return Promise.all(promiseArray);
        }).then((result) => {
            if (result && result.length) {
                let errors = result.filter(item => item === false);
                if (errors && errors.length) {
                    logger.error('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'errors found after CreateQueueL -> SemanticQuery total:', errors.length);
                    return Promise.resolve(false);
                }
            }
            return Promise.resolve(true);
        }).then(result => {
            return mssql_service_1.MSSql.GetEvaluationPlanCognitiveInteractionTask(vcc, evaluationPlanId);
        }).then((result) => {
            if (!result.status)
                throw 'SERVER_ERROR_GET_INTERACTIONS_TASK_FAILED';
            let processInteractions = result?.data?.length ? result.data : [];
            processInteractions = processInteractions.filter(pi => pi.mainTarget === 'ABILITY');
            if (processInteractions?.length) {
                totalAbility += processInteractions.length;
                logger.info('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetEvaluationPlanCognitiveInteractionTask:', processInteractions.length);
                let promiseArray = [];
                processInteractions.forEach((interaction, i) => {
                    interaction.queueId = `${this.GetQueueKeyFromProcessInteraction(interaction)}:${interaction.nluId}:${interaction.mainTarget.toLowerCase()}:${evaluationPlanId}`;
                    promiseArray.push(this.CognitiveInteractionExecute(interaction.mainTarget.toLowerCase(), evaluationPlanId, interaction, currentQueueId, executionCode));
                    if (evaluationPlanNluModels.filter(n => n === interaction.nluId).length === 0)
                        evaluationPlanNluModels.push(interaction.nluId);
                    if (evaluationPlanNluRasaModels.filter(n => n === interaction.nluId).length === 0 && interaction?.cognitiveEngine.includes('RASA'))
                        evaluationPlanNluRasaModels.push(interaction.nluId);
                });
                evaluationPlanNluModels.forEach(n => {
                    billing.AddInsertCognitiveAnalysisDetailToMainFile(vcc, 'EVALUATION_PLAN', evaluationPlanId, processId, n);
                });
                return Promise.all(promiseArray);
            }
            else {
                if (currentQueueId)
                    logger.warn('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetEvaluationPlanCognitiveInteractionTask: 0');
                return Promise.resolve(null);
            }
        }).then((result) => {
            if (result && result.length) {
                let errors = result.filter(item => item === false);
                if (errors && errors.length) {
                    logger.error('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'errors found after InteractionProcessExecute total:', errors.length);
                    return Promise.resolve(false);
                }
            }
            return Promise.resolve(true);
        }).then(result => {
            if (evaluationPlanNluModels.length > 0)
                return this.RunCurrentBatchesRasaModels(vcc, 'EVALUATION_PLAN', evaluationPlanId, evaluationPlanNluRasaModels, executionCode, currentQueueId);
            else
                return Promise.resolve(true);
        }).then(result => {
            let promiseArray = [];
            logger.debug('[ProcessImplementation::ProcessAbility] Creating progress keys for process ' + processId);
            promiseArray.push(redis_service_1.Redis.SaveTotalProgress(currentQueueId, totalAbility));
            promiseArray.push(redis_service_1.Redis.SaveProgress(currentQueueId, totalAbility));
            return Promise.all(promiseArray);
        }).then(result => {
            return Promise.resolve(true);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            let promiseArray = [];
            if (['CANCEL_PROCESS_PENDING', 'CANCEL_PROCESS_FAILED'].includes(errorMessage)) {
                isError = errorMessage === 'CANCEL_PROCESS_PENDING' ? true : false;
                logger.warn('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                promiseArray.push(redis_service_1.Redis.DeleteTotalProgress(currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteProgress(currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteErrorProgress(currentQueueId));
                if (errorMessage === 'CANCEL_PROCESS_FAILED') {
                    logger.debug('[ProcessImplementation::ProcessAbility] UpdateStatusEvaluationPlan ' + evaluationPlanId, 'getting queueId:', currentQueueId);
                    const currentQueueIdTipification = `${vcc}:${processId}:tipification:${evaluationPlanId}`;
                    promiseArray.push(redis_service_1.Redis.DeleteTotalProgress(currentQueueIdTipification));
                    promiseArray.push(redis_service_1.Redis.DeleteProgress(currentQueueIdTipification));
                    promiseArray.push(redis_service_1.Redis.DeleteErrorProgress(currentQueueIdTipification));
                    promiseArray.push(mssql_service_1.MSSql.UpdateStatusEvaluationPlan(vcc, evaluationPlanId, evaluation_plan_model_1.EvaluationPlanStatus.STATUS_FAILED));
                }
            }
            else {
                throw errorMessage;
            }
            return Promise.all(promiseArray);
        }).then(result => {
            if (isError)
                return Promise.resolve(false);
            return Promise.resolve(true);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            if (errorMessage === 'CANCEL_PROCESS_FAILED') {
                logger.warn('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                return Promise.resolve(true);
            }
            else if (errorMessage === 'CANCEL_PROCESS_PENDING') {
                logger.warn('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                return Promise.resolve(false);
            }
            logger.error('[ProcessImplementation::ProcessAbility] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    ProcessEvaluationPlanTipification(vcc, processId, evaluationPlanId, executionCode, currentQueueId) {
        let newQueueId = '';
        let evaluationPlan = null;
        let totalAbility = 0;
        let isError = false;
        let evaluationPlanNluModels = [];
        let evaluationPlanNluRasaModels = [];
        let billing = new billing_implementation_1.BillingImplementation();
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetEvaluationPlan(vcc, evaluationPlanId);
        })
            .then((result) => {
            if (result?.statusDescription === 'FAILED') {
                logger.error('[ProcessImplementation::ProcessEvaluationPlanTipification] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'EvaluationPlan failed, canceling tipification evaluation phase');
                throw 'CANCELLED_BY_EVALUATION_PLAN_FAILED';
            }
            evaluationPlan = result;
            if (['EVALUATION_COMPLETED', 'EVALUATION_COMPLETED_WITH_ERROR'].includes(evaluationPlan.statusDescription)
                &&
                    [11, 12].includes(evaluationPlan.status)
                &&
                    ['A', 'X'].includes(evaluationPlan.evaluation_mode))
                return Promise.resolve(true);
            throw 'CANCELING_BY_EVALUATION_PLAN_IN_PROGRESS';
        })
            .then(_ => {
            return this.GetEvaluationPlanAbilities(vcc, evaluationPlan.id);
        })
            .then(result => {
            return redis_service_1.Redis.SaveEvaluationPlanAbilities(vcc, evaluationPlan.id, result);
        })
            .then(_ => {
            return mssql_service_1.MSSql.GetEvaluationPlanTipificationInteractionTask(vcc, evaluationPlanId);
        })
            .then((result) => {
            if (!result.status)
                throw 'SERVER_ERROR_GET_INTERACTIONS_TASK_FAILED';
            let processInteractions = result?.data?.length ? result.data : [];
            processInteractions = processInteractions.filter(pi => pi.mainTarget === 'TIPIFICATION');
            if (processInteractions?.length) {
                totalAbility += processInteractions.length;
                logger.info('[ProcessImplementation::ProcessEvaluationPlanTipification] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetEvaluationPlanCognitiveInteractionTask:', processInteractions.length);
                let promiseArray = [];
                processInteractions.forEach((interaction, i) => {
                    interaction.queueId = `${this.GetQueueKeyFromProcessInteraction(interaction)}:${interaction.nluId}:${interaction.mainTarget.toLowerCase()}:${evaluationPlanId}`;
                    if (interaction.needRequest) {
                        promiseArray.push(this.CognitiveInteractionExecute(interaction.mainTarget.toLowerCase(), evaluationPlanId, interaction, currentQueueId, executionCode));
                        if (evaluationPlanNluModels.filter(n => n === interaction.nluId).length === 0)
                            evaluationPlanNluModels.push(interaction.nluId);
                        if (evaluationPlanNluRasaModels.filter(n => n === interaction.nluId).length === 0 && interaction?.cognitiveEngine.includes('RASA'))
                            evaluationPlanNluRasaModels.push(interaction.nluId);
                    }
                    else {
                        promiseArray.push(this.TipificationInteractionExecute(interaction.mainTarget.toLowerCase(), evaluationPlanId, interaction, currentQueueId, executionCode));
                    }
                });
                evaluationPlanNluModels.forEach(n => {
                    billing.AddInsertCognitiveAnalysisDetailToMainFile(vcc, 'EVALUATION_PLAN', evaluationPlanId, processId, n);
                });
                return Promise.all(promiseArray);
            }
            else {
                if (currentQueueId)
                    logger.warn('[ProcessImplementation::ProcessEvaluationPlanTipification] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetEvaluationPlanCognitiveInteractionTask: 0');
                return Promise.resolve(null);
            }
        })
            .then((result) => {
            if (result?.length) {
                let errors = result.filter(item => item === false);
                if (errors?.length) {
                    logger.error('[ProcessImplementation::ProcessEvaluationPlanTipification] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'errors found after InteractionProcessExecute total:', errors.length);
                    return Promise.resolve(false);
                }
            }
            return Promise.resolve(true);
        })
            .then(_ => {
            if (evaluationPlanNluModels.length > 0)
                return this.RunCurrentBatchesRasaModels(vcc, 'EVALUATION_PLAN', evaluationPlanId, evaluationPlanNluRasaModels, executionCode, currentQueueId);
            else
                return Promise.resolve(true);
        })
            .then(_ => {
            let promiseArray = [];
            logger.debug('[ProcessImplementation::ProcessEvaluationPlanTipification] Creating progress keys for process ' + processId);
            promiseArray.push(redis_service_1.Redis.SaveTotalProgress(currentQueueId, totalAbility));
            promiseArray.push(redis_service_1.Redis.SaveProgress(currentQueueId, totalAbility));
            return Promise.all(promiseArray);
        })
            .then(result => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            let promiseArray = [];
            if (errorMessage === 'CANCELING_BY_EVALUATION_PLAN_IN_PROGRESS') {
                isError = true;
                logger.warn('[ProcessImplementation::ProcessEvaluationPlanTipification] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                promiseArray.push(redis_service_1.Redis.DeleteTotalProgress(currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteProgress(currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteErrorProgress(currentQueueId));
            }
            else {
                throw errorMessage;
            }
            return Promise.all(promiseArray);
        })
            .then(result => {
            if (isError)
                return Promise.resolve(false);
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            if (errorMessage === 'CANCELLED_BY_EVALUATION_PLAN_FAILED') {
                logger.warn('[ProcessImplementation::ProcessEvaluationPlanTipification] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                return Promise.resolve(true);
            }
            else if (errorMessage === 'CANCELING_BY_EVALUATION_PLAN_IN_PROGRESS') {
                logger.warn('[ProcessImplementation::ProcessEvaluationPlanTipification] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                return Promise.resolve(false);
            }
            logger.error('[ProcessImplementation::ProcessEvaluationPlanTipification] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    ProcessI6UpdateExecute(interactionToUpdate, queueId, executionCode) {
        let currentKey = '';
        let itemExist = false;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (!interactionToUpdate)
                throw 'INTERACTION_IS_NOT_FOUND';
            currentKey = this.GetQueueKeyFromProcessInteraction(interactionToUpdate, 'interaction');
            return redis_service_1.Redis.GetProcessI6Update(queueId, currentKey);
        })
            .then((result) => {
            if (!result) {
                return redis_service_1.Redis.SaveProcessI6Update(queueId, currentKey, interactionToUpdate);
            }
            itemExist = true;
            return Promise.resolve(null);
        })
            .then((result) => {
            if (!itemExist && result) {
                return redis_service_1.Redis.CreateQueueL('I6Update', currentKey);
            }
            return Promise.resolve(null);
        })
            .then((result) => {
            if (!result) {
                if (itemExist) {
                    logger.warn('[ProcessImplementation::ProcessI6UpdateExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'interaction exists, queue process was not inserted from the Interactions queue');
                    return Promise.resolve(true);
                }
                logger.error('[ProcessImplementation::ProcessI6UpdateExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'queue process was not inserted from the Interactions queue');
                return Promise.resolve(false);
            }
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[ProcessImplementation::ProcessI6UpdateExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    ProcessInteractionExecute(processInteraction, queueId, executionCode) {
        let currentKey = '';
        let itemExist = false;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            if (!processInteraction)
                throw 'AUDIO_IS_NOT_FOUND';
            currentKey = this.GetQueueKeyFromProcessInteraction(processInteraction);
            return redis_service_1.Redis.GetProcessInteraction(queueId, currentKey);
        }).then((result) => {
            if (!result) {
                return redis_service_1.Redis.SaveProcessInteraction(queueId, currentKey, processInteraction);
            }
            itemExist = true;
            return Promise.resolve(null);
        }).then((result) => {
            if (!itemExist && result) {
                return redis_service_1.Redis.CreateQueueL('Audios', currentKey);
            }
            return Promise.resolve(null);
        }).then((result) => {
            if (!result) {
                if (itemExist) {
                    logger.warn('[ProcessImplementation::ProcessInteractionExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'audio exists, queue process was not inserted from the Audios queue');
                    return Promise.resolve(true);
                }
                logger.error('[ProcessImplementation::ProcessInteractionExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'queue process was not inserted from the Audios queue');
                return Promise.resolve(false);
            }
            return Promise.resolve(true);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[ProcessImplementation::ProcessInteractionExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    CognitiveInteractionExecute(batchType, typeId, interaction, queueId, executionCode) {
        let currentKey = '';
        let itemExist = false;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (!interaction)
                throw 'INTERACTION_COGNITIVE_NOT_FOUND';
            currentKey = this.GetQueueKeyFromProcessInteraction(interaction);
            currentKey += ':';
            currentKey += interaction.nluId;
            currentKey += ':';
            currentKey += batchType;
            currentKey += ':';
            currentKey += typeId;
            return redis_service_1.Redis.GetCognitiveInteraction(currentKey);
        })
            .then((result) => {
            if (!result) {
                interaction.statusDescription = 'pendingPhrases';
                return redis_service_1.Redis.SaveCognitiveInteraction(currentKey, interaction);
            }
            itemExist = true;
            return Promise.resolve(null);
        })
            .then((result) => {
            if (!itemExist
                &&
                    result)
                return redis_service_1.Redis.DeleteQueue('CognitiveAnalysis', currentKey);
            return Promise.resolve(null);
        })
            .then(_ => {
            if (!itemExist)
                return redis_service_1.Redis.CreateQueueL('CognitiveAnalysis', currentKey);
            return Promise.resolve(null);
        })
            .then((result) => {
            if (!result) {
                if (itemExist) {
                    logger.warn('[ProcessImplementation::CognitiveInteractionExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'audio exists, queue process was not inserted from the Audios queue');
                    return Promise.resolve(true);
                }
                logger.error('[ProcessImplementation::CognitiveInteractionExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'queue process was not inserted from the Audios queue');
                return Promise.resolve(false);
            }
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[ProcessImplementation::CognitiveInteractionExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    TipificationInteractionExecute(batchType, typeId, interaction, queueId, executionCode) {
        let currentKey = '';
        let itemExist = false;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (!interaction)
                throw 'INTERACTION_COGNITIVE_NOT_FOUND';
            currentKey = `${this.GetQueueKeyFromProcessInteraction(interaction)}:${interaction.nluId}:${batchType}:${typeId}`;
            return redis_service_1.Redis.GetTipificationInteraction(currentKey);
        })
            .then((result) => {
            if (!result) {
                interaction.statusDescription = 'pendingAnalysis';
                return redis_service_1.Redis.SaveTipificationInteraction(currentKey, interaction);
            }
            itemExist = true;
            return Promise.resolve(null);
        })
            .then((result) => {
            if (!itemExist
                &&
                    result
                &&
                    interaction.mainTarget === 'TIPIFICATION')
                return redis_service_1.Redis.CreateQueueL('TipificationAnalysis', currentKey);
            return Promise.resolve(null);
        })
            .then((result) => {
            if (!result) {
                if (itemExist) {
                    logger.warn('[ProcessImplementation::TipificationInteractionExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'audio exists, queue process was not inserted from the Audios queue');
                    return Promise.resolve(true);
                }
                logger.error('[ProcessImplementation::TipificationInteractionExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'queue process was not inserted from the Audios queue');
                return Promise.resolve(false);
            }
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[ProcessImplementation::TipificationInteractionExecute] executionCode:', executionCode, 'getting queueId:', queueId, 'id:', currentKey, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    GetStudyTemplateThemes(vcc, templateId) {
        let templateTemplateThemes = [];
        return promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetTemplateThemes(vcc, templateId);
        })
            .then(result => {
            templateTemplateThemes = result;
            let promiseArray = [];
            templateTemplateThemes.map(ttt => {
                promiseArray.push(template_service_1.GetTemplateItems(vcc, ttt.id, true));
            });
            return Promise.all(promiseArray);
        })
            .then(result => {
            result.map((r, i) => {
                templateTemplateThemes[i].items = r;
            });
            return Promise.resolve(templateTemplateThemes);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_GET_STUDY_THEMES_FAILED';
            logger.error('[ProcessImplementation::GetStudyTemplateThemes] Error: ' + JSON.stringify(error));
            return Promise.reject(errorMessage);
        });
    }
    GetEvaluationPlanAbilities(vcc, evaluationPlanId) {
        let evaluationFormatAbilities = [];
        return promises_shared_1.DummyPromise()
            .then(_ => {
            let promiseArray = [];
            promiseArray.push(mssql_service_1.MSSql.GetEvaluationPlanAbilityEvaluationFormats(vcc, evaluationPlanId));
            promiseArray.push(mssql_service_1.MSSql.GetEvaluationPlanScores(vcc, evaluationPlanId));
            promiseArray.push(mssql_service_1.MSSql.GetEvaluationPlanAbilitiesQueries(vcc, evaluationPlanId));
            promiseArray.push(mssql_service_1.MSSql.GetEvaluationPlanAbilitiesQueriesEntities(vcc, evaluationPlanId));
            return Promise.all(promiseArray);
        })
            .then(result => {
            evaluationFormatAbilities = result[0];
            if (evaluationFormatAbilities?.length) {
                if (result[2]?.length) {
                    result[2].forEach(aq => {
                        aq.entities = result[3]?.length ? result[3].filter(aqe => aqe.abilityQueryId === aq.id) : [];
                    });
                }
                evaluationFormatAbilities.forEach(a => {
                    a.scores = result[1]?.length ? result[1].filter(s => s.idAbility === a.idAbility) : [];
                    a.queries = result[2]?.length ? result[2].filter(aq => aq.abilityId === a.idAbility) : [];
                });
            }
            return Promise.resolve(evaluationFormatAbilities);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_GET_EVALUATIONPLAN_ABILITIES_FAILED';
            logger.error('[ProcessImplementation::GetEvaluationPlanAbilities] Error: ' + JSON.stringify(error));
            return Promise.reject(errorMessage);
        });
    }
    GetQueueKeyFromProcessInteraction(interaction, keyType = 'detail') {
        if (!interaction)
            throw 'INTERACTION_NOT_FOUND';
        let currentKey = '';
        let interactionId = '';
        if (interaction.interactionType === 'CALL') {
            interactionId = JSON.parse(JSON.stringify(interaction.id));
        }
        else {
            interactionId = interaction.interactionType + '_' + interaction.campaign + '_' + interaction.folder + '_' + JSON.parse(JSON.stringify(interaction.id));
        }
        currentKey = interaction.vcc;
        currentKey += ':';
        currentKey += interaction.processId;
        currentKey += ':';
        currentKey += interaction.interactionType;
        currentKey += ':';
        currentKey += interactionId.split(':').join('|');
        if (keyType === 'detail') {
            currentKey += ':';
            currentKey += interaction.section ? interaction.section : '0';
            currentKey += ':';
            currentKey += interaction.part ? interaction.part : '0';
        }
        return currentKey;
    }
    RunCurrentBatchesRasaModels(vcc, batchTypeName, batchTypeId, nlus, executionCode, currentQueueId) {
        let key = `${batchTypeName}::${batchTypeId}`;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return redis_service_1.Redis.GetCurrentBatchesRasaModels();
        })
            .then(result => {
            let appIds = [];
            if (result && result[key]?.length && typeof result[key] === 'string') {
                try {
                    appIds = JSON.parse(result[key]);
                }
                catch (error) {
                    logger.error('[ProcessImplementation::RunCurrentBatchesRasaModels] Error parsing: ' + JSON.stringify(error));
                    appIds = [];
                }
            }
            nlus.forEach(n => {
                let nluKey = `${vcc}::${n}`;
                if (!appIds.includes(nluKey))
                    appIds.push(n = nluKey);
            });
            return redis_service_1.Redis.SaveCurrentBatchesRasaModels(key, appIds);
        })
            .then(result => {
            logger.info('[ProcessImplementation::RunCurrentBatchesRasaModels] Result: ' + result);
            return Promise.resolve(true);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'SERVER_ERROR_RUN_CURRENT_BATCHES_NLUS_FAILED';
            logger.error('[ProcessImplementation::RunCurrentBatchesRasaModels] Error: ' + JSON.stringify(error));
            return Promise.reject(errorMessage);
        });
    }
}
exports.ProcessImplementation = ProcessImplementation;
