"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RASAAPIServiceImplementation = void 0;
const log4js = require("log4js");
const config_1 = require("../config/config");
const promises_shared_1 = require("../shared/promises.shared");
const request_shared_1 = require("../shared/request.shared");
const utilities_1 = require("../shared/utilities");
const mssql_service_1 = require("../db/mssql.service");
const utterance_service_1 = require("../services/utterance.service");
const platform_service_1 = require("../services/platform.service");
let logger = log4js.getLogger('Task');
class RASAAPIServiceImplementation {
    GetCognitiveEngineName() {
        return 'RASAAPI';
    }
    GetCognitiveEngineCharsPerPhrase() {
        return config_1.COGNITIVE_SERVICES_RASA_API_CHARS_PER_PHRASE;
    }
    LoadAzureAccounts(subscriptionRegion, subscriptionKey, data) {
        return null;
    }
    ModelQuery(subscriptionRegion, subscriptionKey, appId, query, processAudios, nluName, resolve, task) {
        let apiKey = config_1.RASA_API_KEY;
        let appIdRasaApi = appId;
        let uri = config_1.RASA_API_URL.replace('/api', '');
        let data = {
            "message": query
        };
        let postOptions = {
            uri: `https://${uri}/query/${appIdRasaApi}`,
            body: data,
            headers: {
                'apiKey': apiKey
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[RASAAPIServiceImplementation::ModelQuery] getting postOptions:', postOptions, 'appId:', appId, 'query:', query, 'nluName:', nluName);
        return request_shared_1.SimplePostRequest(postOptions)
            .then((response) => {
            logger.info('[RASAAPIServiceImplementation::ModelQuery] getting postOptions:', postOptions, 'appId:', appId, 'query:', query, 'result:', response);
            return Promise.resolve(response);
        })
            .then((result) => {
            logger.info('[RASAAPIServiceImplementation::ModelQuery] getting postOptions:', postOptions, 'appId:', appId, 'query:', query, 'result:', result);
            return Promise.resolve(this.FormatModelQueryResponse(result));
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[RASAAPIServiceImplementation::ModelQuery] getting error:', errorMessage, 'appId:', appId, 'query:', query, 'nluName:', nluName);
            return Promise.reject(err);
        });
    }
    FormatModelQueryResponse(originalResponse) {
        let formatedData;
        let originalIntents = originalResponse.intent_ranking;
        let originalEntities = originalResponse.entities;
        let originalQuery = originalResponse.text;
        let formatedIntents = [];
        let formatedEntities = [];
        let formatedQuery;
        formatedIntents = originalIntents.map(oi => {
            return {
                intent: oi.name,
                score: oi.confidence
            };
        });
        formatedEntities = originalEntities;
        formatedQuery = originalQuery;
        formatedData = {
            intents: formatedIntents,
            entities: formatedEntities,
            query: formatedQuery
        };
        return formatedData;
    }
    DeleteAppIntent(subscriptionRegion, subscriptionKey, appId, intentId, vcc) {
        return promises_shared_1.DummyPromise()
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        }).then(result => {
            logger.info('[RASAServiceImplementation::DeleteAppIntent] Getting result:', result, 'after GetVersion');
            return mssql_service_1.MSSql.DeleteIntent(vcc, appId, intentId);
        })
            .then(result => {
            return utterance_service_1.GetUtterances(vcc, appId, null, null, null, true);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAAPIServiceImplementation::DeleteAppIntent] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    DeleteAppUtterance(subscriptionRegion, subscriptionKey, appId, intentId, utteranceId, vcc) {
        return promises_shared_1.DummyPromise()
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        }).then(result => {
            logger.info('[RASAServiceImplementation::AddIntent] Getting result:', result, 'after GetVersion');
            return mssql_service_1.MSSql.DeleteUtterance(vcc, appId, intentId, utteranceId);
        })
            .then(result => {
            return utterance_service_1.GetUtterances(vcc, appId, null, null, null, true);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::DeleteUtterance] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    TrainApp(subscriptionRegion, subscriptionKey, appId, vcc) {
        let appIdRasaApi = appId;
        let nluObjectVersion = {
            name: "debug_speechanalytics",
            active: true,
            nlu: {
                intents: []
            }
        };
        return promises_shared_1.DummyPromise()
            .then(result => {
            return this.GenerateNluObjectVersion(appId, vcc);
        }).then(result => {
            nluObjectVersion.nlu.intents = result;
            if (!result || result?.length === 0)
                throw 'SERVER_ERROR_GENERATE_NLU_OBJECT_VERSION_RASAAPI_FAILED';
            let data = nluObjectVersion;
            let apiKey = config_1.RASA_API_KEY;
            let postOptions = {
                uri: `https://${config_1.RASA_API_URL}/application/${appIdRasaApi}/version`,
                body: data,
                headers: {
                    'apiKey': apiKey,
                },
                retryProtocol: false,
                timeout: 60000
            };
            logger.info('[RASAAPIServiceImplementation::TrainApp] getting postOptions:', postOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'data:', data);
            return request_shared_1.SimplePostRequest(postOptions)
                .then((result) => {
                logger.info('[RASAAPIServiceImplementation::TrainApp] getting response successfully response:', result);
                return this.TrainWithVersion(result.id);
            });
        }).then(result => {
            return Promise.resolve(result);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::TrainApp] Error: ', error);
            if (error === 'SERVER_ERROR_MISSING_UTTERANCES_TO_TRAIN')
                return Promise.resolve({ status: 'Queued', statusId: 9 });
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GetTrainApp(subscriptionRegion, subscriptionKey, appId, trainId) {
        let getOptions = {
            uri: `https://${config_1.RASA_API_URL}/train/${trainId}`,
            headers: {
                'apiKey': config_1.RASA_API_KEY,
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[RASAAPIServiceImplementation::GetTrainApp] getting getOptions:', getOptions, 'trainId:', trainId);
        return request_shared_1.SimpleGetRequest(getOptions)
            .then((response) => {
            logger.info('[RASAAPIServiceImplementation::GetTrainApp] getting response:', getOptions, 'result:', response);
            return Promise.resolve(response);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[RASAAPIServiceImplementation::GetTrainApp] getting error:', errorMessage, 'trainId:', trainId);
            return Promise.reject(err);
        });
    }
    PublishApp(subscriptionRegion, subscriptionKey, appId, isStaging, trainVersion) {
        let apiKey = config_1.RASA_API_KEY;
        let appIdRasaApi = appId;
        let data = {
            "active": true
        };
        let putOptions = {
            uri: `https://${config_1.RASA_API_URL}/application/${appIdRasaApi}/version/${trainVersion}`,
            body: data,
            headers: {
                'apiKey': apiKey
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[RASAAPIServiceImplementation::PublishApp] getting postOptions:', putOptions, 'apikey:', apiKey, 'appId:', appId);
        return request_shared_1.SimplePutRequest(putOptions)
            .then((response) => {
            logger.info('[RASAAPIServiceImplementation::PublishApp] getting response successfully apikey:', apiKey, 'appId:', appId, 'response', response);
            return Promise.resolve(response);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[RASAAPIServiceImplementation::PublishApp] getting error:', errorMessage, 'apikey:', apiKey, 'appId:', appId);
            return Promise.reject(err);
        });
    }
    GetManagerToken(info) {
        return null;
    }
    UnAssignSubscriptionKeyOfVCC(subscriptionRegion, subscriptionKey, appId, data) {
        return null;
    }
    AssignSubscriptionKeyOfVCC(subscriptionRegion, subscriptionKey, appId, data) {
        return null;
    }
    GetCultures(subscriptionRegion, subscriptionKey) {
        let cultures = [];
        cultures.push({ name: "Chinese", code: "zh" });
        cultures.push({ name: "Dutch", code: "nl" });
        cultures.push({ name: "English", code: "en" });
        cultures.push({ name: "French", code: "fr" });
        cultures.push({ name: "German", code: "de" });
        cultures.push({ name: "Italian", code: "it" });
        cultures.push({ name: "Portuguese", code: "pt" });
        cultures.push({ name: "Spanish", code: "es" });
        return Promise.resolve(cultures.filter(c => config_1.COGNITIVE_SERVICES_RASA_API_AVAILABLE_LANGUAGES.includes(c.code)));
    }
    AddApp(subscriptionRegion, subscriptionKey, nlu) {
        let data = { name: nlu.name, language: nlu.culture };
        let apiKey = config_1.RASA_API_KEY;
        let postOptions = {
            uri: `https://${config_1.RASA_API_URL}/application`,
            body: data,
            headers: {
                'apiKey': apiKey,
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[RASAAPIServiceImplementation::AddApp] getting postOptions:', postOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'nlu:', nlu);
        return request_shared_1.SimplePostRequest(postOptions)
            .then((response) => {
            logger.info('[RASAAPIServiceImplementation::AddApp] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'nlu:', nlu);
            return Promise.resolve(response);
        })
            .then((result) => {
            logger.info('[RASAAPIServiceImplementation::AddApp] getting response successfully RasaApiAppId:', result.id);
            return Promise.resolve(result.id);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[RASAAPIServiceImplementation::AddApp] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'nlu:', nlu);
            return Promise.reject(err);
        });
    }
    DeleteApp(subscriptionRegion, subscriptionKey, nlu) {
        let rasaAppId = nlu.appId;
        let apiKey = config_1.RASA_API_KEY;
        let deleteOptions = {
            uri: `https://${config_1.RASA_API_URL}/application/${rasaAppId}`,
            headers: {
                'apiKey': apiKey
            },
            timeout: 60000
        };
        logger.info('[RASAAPIServiceImplementation::DeleteApp] getting deleteOptions:', deleteOptions, 'appId:', nlu.appId);
        return request_shared_1.SimpleDeleteRequest(deleteOptions)
            .then((response) => {
            logger.info('[RASAAPIServiceImplementation::DeleteApp] getting response successfully', 'appId:', nlu.appId);
            return response;
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[RASAAPIServiceImplementation::DeleteApp] getting error:', errorMessage, 'appId:', nlu.appId);
            return Promise.resolve(err);
        });
    }
    GetAppsList(subscriptionRegion, subscriptionKey) {
        logger.info('[RASAAPIServiceImplementation::GetAppsList] getting subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
        return this.GetResultsFromPaginatedRequest(subscriptionRegion, subscriptionKey, `https://${config_1.RASA_API_URL}/application/`, 0)
            .then((response) => {
            logger.info('[RASAAPIServiceImplementation::GetAppsList] getting response successfully response:', response);
            return Promise.resolve(response);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[RASAAPIServiceImplementation::GetAppsList] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
            return Promise.reject(err);
        });
    }
    TrainWithVersion(trainVersion) {
        logger.info('[RASAAPIServiceImplementation::TrainWithVersion] getting trainVersion:', trainVersion);
        let data = '';
        let apiKey = config_1.RASA_API_KEY;
        let postOptions = {
            uri: `https://${config_1.RASA_API_URL}/train/${trainVersion}`,
            body: data,
            headers: {
                'apiKey': apiKey,
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[RASAAPIServiceImplementation::TrainWithVersion] getting postOptions:', postOptions);
        return request_shared_1.SimplePostRequest(postOptions)
            .then((response) => {
            logger.info('[RASAAPIServiceImplementation::TrainWithVersion] getting response successfully response:', response);
            return response;
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[RASAAPIServiceImplementation::TrainWithVersion] getting error:', errorMessage);
            return Promise.reject(err);
        });
    }
    GetAppById(appId, subscriptionRegion, subscriptionKey) {
        let appIdRasaApi = appId;
        let getOptions = {
            uri: `https://${config_1.RASA_API_URL}/application/${appIdRasaApi}`,
            headers: {
                'apiKey': config_1.RASA_API_KEY,
            },
            retryProtocol: false,
            timeout: 60000,
            validateErrorResponse: true
        };
        logger.info('[RASAAPIServiceImplementation::GetAppById] getting getOptions:', getOptions, 'appId:', appIdRasaApi);
        return request_shared_1.SimpleGetRequest(getOptions)
            .then((response) => {
            logger.info('[RASAAPIServiceImplementation::GetAppById] getting response:', getOptions, 'result:', response);
            return Promise.resolve(response);
        }).catch((err) => {
            let error = typeof err;
            logger.debug("tipo de error", error);
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description || err.error);
            logger.error('[RASAAPIServiceImplementation::GetAppById] getting error:', err);
            logger.error('[RASAAPIServiceImplementation::GetAppById] getting error2:', err.error);
            logger.error('[RASAAPIServiceImplementation::GetAppById] getting error3:', errorMessage);
            if (errorMessage.includes('Application not found'))
                return Promise.resolve(errorMessage);
            return Promise.reject(errorMessage);
        });
    }
    AddIntent(subscriptionRegion, subscriptionKey, appId, name) {
        return promises_shared_1.DummyPromise()
            .then(result => {
            return Promise.resolve(utilities_1.guid());
        });
    }
    AddUtterance(subscriptionRegion, subscriptionKey, appId, utterance, vcc) {
        let intentName = utterance.intent;
        let text = utterance.name;
        let entityLabels = utterance.entityLabels ? utterance.entityLabels : [];
        let savedUtterance = null;
        let savedUtteranceEntities = [];
        return promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetUtterance(vcc, appId, intentName, text);
        })
            .then(result => {
            savedUtterance = result;
            if (savedUtterance) {
                utterance.id = savedUtterance.id;
            }
            else {
                utterance.id = utilities_1.guid();
            }
        })
            .then(result => {
            if (savedUtterance) {
                return mssql_service_1.MSSql.GetUtteranceEntities(vcc, utterance.id);
            }
            else {
                return Promise.resolve([]);
            }
        })
            .then(result => {
            if (result)
                savedUtteranceEntities = result;
            let promiseArray = [];
            entityLabels.map(el => {
                if (!savedUtteranceEntities.find(sue => sue.utteranceId === utterance.id && sue.entityName === el.entityName && sue.start === el.start && sue.end === el.end)) {
                    promiseArray.push(mssql_service_1.MSSql.SaveUtteranceEntity(vcc, utterance.id, el.entityName, el.start, el.end, el.startTokenIndex, el.endTokenIndex));
                }
            });
            savedUtteranceEntities.map(sue => {
                if (!entityLabels.find(el => el.utteranceId === utterance.id && el.entityName === sue.entityName && el.start === sue.start && el.end === sue.end)) {
                    promiseArray.push(mssql_service_1.MSSql.DeleteUtteranceEntity(vcc, utterance.id, sue.entityName, sue.start, sue.end, sue.startTokenIndex, sue.endTokenIndex));
                }
            });
            return Promise.all(promiseArray);
        })
            .then(result => {
            return Promise.resolve({
                UtteranceText: utterance.name,
                ExampleId: savedUtterance ? savedUtterance.id : utilities_1.guid()
            });
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAAPIServiceImplementation::AddUtterance] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GetAppVersion(subscriptionRegion, subscriptionKey, appId, vcc) {
        return promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, appId);
        }).then(result => {
            return Promise.resolve(result);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAAPIServiceImplementation::GetAppVersion] Error: ', error);
            return Promise.reject(errorMessage);
        });
    }
    GetIntentsList(subscriptionRegion, subscriptionKey, appId, vcc) {
        return promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetIntents(vcc, appId, null, null, null);
        })
            .then(result => {
            let res = result ? result : [];
            let arrRes = [];
            res.map(r => {
                arrRes.push({
                    id: r.intentId,
                    name: r.name
                });
            });
            return Promise.resolve(arrRes);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetIntentsList] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GetIntentsListDemo(subscriptionRegion, subscriptionKey, appId, resolve, reject, vcc) {
        return promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetIntents(vcc, appId, null, null, null);
        })
            .then(result => {
            let res = result ? result : [];
            let arrRes = [];
            res.map(r => {
                arrRes.push({
                    id: r.intentId,
                    name: r.name
                });
            });
            return Promise.resolve(arrRes);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAAPIServiceImplementation::GetIntentsListDemo] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GetUtterancesList(subscriptionRegion, subscriptionKey, appId, vcc) {
        let utterances = [];
        return promises_shared_1.DummyPromise()
            .then(result => {
            return utterance_service_1.GetUtterances(vcc, appId, null, null, null, true);
        })
            .then(result => {
            if (result)
                utterances = result;
            return Promise.resolve(utterances);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetUtterancesList] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GetUtterancesFromMD(appId) {
        return promises_shared_1.DummyPromise()
            .then(result => {
            return Promise.resolve([]);
        });
    }
    AddSimpleEntity(subscriptionRegion, subscriptionKey, appId, entity) {
        return promises_shared_1.DummyPromise()
            .then(result => {
            return Promise.resolve(utilities_1.guid());
        });
    }
    RenameEntity(subscriptionRegion, subscriptionKey, appId, entityId, entityName) {
        return null;
    }
    GetEntitiesList(subscriptionRegion, subscriptionKey, appId, vcc) {
        return promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetEntities(vcc, appId, '', '', '');
        });
    }
    GetEntity(subscriptionRegion, subscriptionKey, appId, entityId) {
        return null;
    }
    DeleteAppEntity(subscriptionRegion, subscriptionKey, appId, entityId) {
        return null;
    }
    GetAllIntentsFromRASAContent(content) {
        return null;
    }
    GetAllUtterancesFromRASAContent(content) {
        return null;
    }
    ExportModel(nlu, res) {
        let installationId = '';
        let modelFolder = null;
        let modelToExport = {};
        let intentsToExport = [];
        let entitiesToExport = [];
        let utteranceEntitiesToExport = [];
        return promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetNLUByAppId(nlu.vcc, nlu.appId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_NLU_NOT_FOUND';
            modelToExport = {
                name: result.name,
                culture: result.culture,
                description: result.description,
                cognitiveEngine: this.GetCognitiveEngineName()
            };
            let promiseArray = [];
            promiseArray.push(mssql_service_1.MSSql.GetIntents(nlu.vcc, nlu.appId, null, null, null));
            promiseArray.push(mssql_service_1.MSSql.GetUtterances(nlu.vcc, nlu.appId, null));
            promiseArray.push(mssql_service_1.MSSql.GetEntities(nlu.vcc, nlu.appId, '', '', null));
            return Promise.all(promiseArray);
        })
            .then(result => {
            if (!result[0])
                throw 'SERVER_ERROR_NLU_INTENTS_NOT_FOUND';
            if (!result[1])
                throw 'SERVER_ERROR_NLU_UTTERANCES_NOT_FOUND';
            if (!result[2])
                throw 'SERVER_ERROR_NLU_ENTITIES_NOT_FOUND';
            let promiseArray = [];
            result[0].map(i => {
                let intentUtterances = [];
                result[1].filter(r => r.intentId === i.intentId).map(u => {
                    intentUtterances.push({
                        id: u.id,
                        text: u.text
                    });
                    promiseArray.push(mssql_service_1.MSSql.GetUtteranceEntities(nlu.vcc, u.id));
                });
                intentsToExport.push({
                    id: i.intentId,
                    name: i.name,
                    color: i.color,
                    utterances: intentUtterances
                });
            });
            result[2].map(e => {
                entitiesToExport.push({
                    id: utilities_1.guid(),
                    name: e.name,
                    type: e.type
                });
            });
            modelToExport['intents'] = intentsToExport;
            modelToExport['entities'] = entitiesToExport;
            return Promise.all(promiseArray);
        })
            .then(result => {
            result.map(r => {
                r.map(ue => {
                    utteranceEntitiesToExport.push({
                        utteranceId: ue.utteranceId,
                        entityName: ue.entityName,
                        start: ue.start,
                        end: ue.end,
                        startTokenIndex: ue.startTokenIndex,
                        endTokenIndex: ue.endTokenIndex
                    });
                });
            });
            modelToExport['utteranceEntities'] = utteranceEntitiesToExport;
            return Promise.resolve(true);
        })
            .then(result => {
            modelToExport.intents.map(i => {
                i.utterances.map(u => {
                    let newId = utilities_1.guid();
                    let oldId = u.id;
                    u.id = newId;
                    modelToExport.utteranceEntities.filter(ue => ue.utteranceId === oldId).map(ue => { ue.utteranceId = newId; });
                });
            });
            return Promise.resolve(modelToExport);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::ExportModel] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GetResultsFromPaginatedRequest(subscriptionRegion, subscriptionKey, url, page) {
        page = page ? page : 0;
        let allResults = [];
        let getOptions = {
            uri: `${url}?page=${page}`,
            headers: {
                'apiKey': config_1.RASA_API_KEY
            },
            retryProtocol: false,
            timeout: 60000
        };
        return request_shared_1.SimpleGetRequest(getOptions)
            .then((result) => {
            if (!result)
                throw 'SERVER_ERROR_GET_RASA_API_ITEMS_SERVICE_FAILED';
            let totalRecords = result.count;
            let pageSize = result.pageSize;
            let maxRecordsToShow = (page + 1) * pageSize;
            if (maxRecordsToShow <= totalRecords) {
                allResults = result.data;
                return this.GetResultsFromPaginatedRequest(subscriptionRegion, subscriptionKey, url, page + 1);
            }
            else {
                return Promise.resolve(result.data);
            }
        })
            .then(result => {
            logger.info('[RASAAPIServiceImplementation::GetResultsFromPaginatedRequest] getting result:', result, 'page:', page, 'totalRecords', 'getoptions:', getOptions);
            allResults = allResults.concat(result);
            return Promise.resolve(allResults);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[RASAAPIServiceImplementation::GetResultsFromPaginatedRequest] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
            return Promise.reject(err);
        });
    }
    RunApp(appId, task) {
        return null;
    }
    StopApp(appId, port, task) {
        return null;
    }
    GenerateNluObjectVersion(appId, vcc) {
        let installationId = '';
        let modelFolder = '';
        let newNluTextContent = '';
        return promises_shared_1.DummyPromise()
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_CREATE_NLU_TEMPORARY_FOLDER';
            return this.GetNluContentFromDatabase(appId, vcc);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_GENERATE_NLU_CONTENT';
            newNluTextContent = result;
            return result;
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GenerateNluMd] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GetNluContentFromDatabase(appId, vcc) {
        let nluIntents = [];
        let nluUtteranceObjects = [];
        let nluObjectVersionContent = null;
        return promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetIntents(vcc, appId, null, null, null);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_MISSING_INTENTS_TO_TRAIN';
            nluIntents = result;
            return utterance_service_1.GetUtterances(vcc, appId, null, null, null, true);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_MISSING_UTTERANCES_TO_TRAIN';
            nluUtteranceObjects = result;
            nluObjectVersionContent = this.GenerateNluObjetVersionContent(nluIntents, nluUtteranceObjects);
            return Promise.resolve(nluObjectVersionContent);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetContentFromDatabase] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GenerateNluObjetVersionContent(intents, utterances) {
        let nluContent = '';
        let intentsObj = [];
        let intentDummy = {
            "action": "None",
            "examples": [
                {
                    "raw": "xxxxx",
                    "data": []
                }
            ]
        };
        intents.map(i => {
            let intentObj = {
                action: i.name,
                examples: []
            };
            let intentBlockText = `## intent:${i.intentId}\n`;
            utterances.filter(u => u.intentId === i.intentId).map(u => {
                let labeledUtterance = null;
                let entities = [];
                let uteranceObj = {
                    id: u.id,
                    raw: u.text,
                    data: []
                };
                if (u.entityLabels && u.entityLabels.length) {
                    u.entityLabels.forEach(el => {
                        entities.push({
                            "value": "entity-" + el.entityName,
                            "entity": el.entityName,
                            "start": el.start,
                            "end": el.entityName
                        });
                    });
                    uteranceObj.data = entities;
                }
                intentObj.examples.push(uteranceObj);
                intentBlockText += `- ${labeledUtterance}\n`;
            });
            intentBlockText += '\n';
            nluContent += intentBlockText;
            intentsObj.push(intentObj);
        });
        intentsObj.push(intentDummy);
        return intentsObj;
    }
    GenerateLabeledUtterance(utteranceText, entityLabels) {
        let labeledUtterance = utteranceText;
        let addedText = '';
        entityLabels.map(el => {
            let splittedText = labeledUtterance.split('');
            labeledUtterance = '';
            splittedText.map((st, i) => {
                if (i === el.start + addedText.length) {
                    labeledUtterance += '[' + st;
                }
                else if (i === el.end + addedText.length) {
                    labeledUtterance += st + `](${el.entityName})`;
                }
                else {
                    labeledUtterance += st;
                }
            });
            addedText += `[](${el.entityName})`;
        });
        return labeledUtterance;
    }
}
exports.RASAAPIServiceImplementation = RASAAPIServiceImplementation;
