"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SemanticAnalysisDownloadImplementation = void 0;
const config_1 = require("../config/config");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const promises_shared_1 = require("../shared/promises.shared");
const log4js = require("log4js");
const moment = require("moment");
const billing_batch_model_1 = require("../models/billing-batch.model");
const billing_implementation_1 = require("./billing.implementation");
const file_manager_1 = require("../shared/file-manager");
const idatha_service_1 = require("../services/idatha.service");
const cognitive_analysis_implementation_1 = require("./cognitive-analysis.implementation");
const idatha_model_1 = require("../models/idatha.model");
const request_shared_1 = require("../shared/request.shared");
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
const CURRENT_IMPLEMENTATION = 'SemanticAnalysisDownloadImplementation';
class SemanticAnalysisDownloadImplementation {
    constructor() {
        this.errorData = new billing_batch_model_1.BatchError();
        this.queueBase = 'SemanticAnalysisDownload';
    }
    Execute(executionCode, rateRecursive) {
        logger.info(`[SemanticAnalysisDownloadImplementation::Execute] executionCode: ${executionCode} executing wave #${rateRecursive}`);
        let executeIndex = 1;
        let ratio = config_1.RATIO_SEMANTIC_ANALYSIS_DOWNLOAD;
        let executeResult = true;
        let currentBlockPromise = null;
        return promises_shared_1.SleepPromise(config_1.SLEEP_SEMANTIC_ANALYSIS)
            .then(_ => {
            currentBlockPromise = 'BLOCK_01_GET_FROM_REDIS';
            return redis_service_1.Redis.GetLengthFromList(this.queueBase);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_02_GET_QUEUE_ITEMS';
            if (!result)
                throw 'SERVER_INFO_REDIS_KEYS_FOR_SEMANTIC_ANALYSIS_DOWNLOAD_NOT_FOUND';
            return redis_service_1.Redis.LRange(this.queueBase, 0, ratio - 1);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_03_SEMANTIC_ANALYSIS_DOWNLOAD_IMPL';
            ratio = result && result.length <= ratio ? result.length : 0;
            let promiseArray = [];
            while (ratio >= executeIndex) {
                promiseArray.push(this.SemanticAnalysisDownloadExecute(executionCode));
                executeIndex++;
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then((results) => {
            currentBlockPromise = 'BLOCK_04_GETTING_KEYS_FOR_RECURSIVITY';
            if (results.filter(item => item === false).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange(this.queueBase, 0, ratio - 1);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_05_RELAUNCH_SEMANTIC_ANALYSIS_UPLOAD_IMP';
            if (result?.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1);
            return Promise.resolve(executeResult);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_06_RESOLVING_IMPL';
            if (rateRecursive === config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS_DOWNLOAD)
                logger.info(`[SemanticAnalysisDownloadImplementation::Execute] executionCode: ${executionCode} Finishing execution wave #${rateRecursive}`);
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            if (errorMessage.startsWith('SERVER_INFO_'))
                logger.info(`[SemanticAnalysisDownloadImplementation::Execute] executionCode: ${executionCode}, Block: ${currentBlockPromise}, getting info:`, errorMessage);
            else
                logger.error(`[SemanticAnalysisDownloadImplementation::Execute] executionCode: ${executionCode}, Block: ${currentBlockPromise}, getting error:`, errorMessage);
            return Promise.resolve(false);
        });
    }
    SemanticAnalysisDownloadExecute(executionCode) {
        let storage = '';
        let currentQueueId = '';
        let processInteraction = null;
        let vcc = '';
        let processId = '';
        let interactionType = '';
        let interactionId = '';
        let interactionSection = 0;
        let interactionPart = 0;
        return redis_service_1.Redis.RPopLPush(this.queueBase, `${this.queueBase}Execute`)
            .then((queueId) => {
            if (queueId?.length > 0) {
                currentQueueId = queueId;
                logger.info('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                let item = queueId.split(':');
                if (item?.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    interactionId = item[3];
                    interactionSection = Number(item[4]);
                    interactionPart = Number(item[5]);
                    storage = vcc + ':' + processId;
                    logger.info('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                    return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
                }
                else {
                    logger.error('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return Promise.resolve(null);
        })
            .then((processInteractionResult) => {
            if (processInteractionResult) {
                processInteraction = processInteractionResult;
                logger.info('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1, status:', processInteraction.statusDescription);
                if (processInteraction.statusDescription === 'pendingDownloadJSON') {
                    let validTime = Number(moment().format('x'));
                    let nextExecutionTime = Number(processInteraction.uploadedDate);
                    nextExecutionTime = nextExecutionTime + config_1.NEXT_EXECUTION_TIME_SEMANTIC_ANALYSIS;
                    if (validTime > nextExecutionTime) {
                        return this.DownloadJSON(processInteraction, currentQueueId, executionCode);
                    }
                    else {
                        logger.info('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_EXECUTION', nextExecutionTime, 'CURRENT_TIME', validTime);
                    }
                }
            }
            if (!processInteraction && currentQueueId) {
                logger.warn('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                if (!processInteraction) {
                    redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId);
                    throw 'SERVER_ERROR_INTERACTION_DATA_NOT_FOUND_ON_STORAGE';
                }
            }
            return Promise.resolve(null);
        })
            .then((result) => {
            if (result)
                processInteraction = result;
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return Promise.resolve(null);
        })
            .then((result) => {
            let promiseArray = [];
            let billing = new billing_implementation_1.BillingImplementation();
            if (result
                &&
                    processInteraction?.status === 6
                &&
                    processInteraction?.statusDescription === 'ready') {
                logger.info('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FINISHED');
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'ValidChar', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'ValidWords', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'TotalPhrases', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'CharsPerPhrase', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'IdathaApi', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 1, 'SUCCED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId));
            }
            else if (result
                &&
                    processInteraction?.status === 6
                &&
                    processInteraction?.statusDescription === 'pendingConvertAudio'
                &&
                    processInteraction?.retry < config_1.RETRY_SEMANTIC_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.InsertQueue('SemanticAnalysisConvertAudio', currentQueueId, config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS_DOWNLOAD * config_1.RATIO_SEMANTIC_ANALYSIS_DOWNLOAD * 2));
                promiseArray.push(redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId));
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                let interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId));
            }
            else {
                logger.warn('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(_ => {
            let promiseArray = [];
            if (processInteraction?.status === 6
                &&
                    processInteraction?.statusDescription === 'ready') {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
            }
            else if (processInteraction?.retry >= config_1.RETRY_SEMANTIC_ANALYSIS) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, processInteraction.batchTypeId, processInteraction.batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Catching error: ' + errorMessage);
            if (processInteraction)
                logger.error('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage, ' vcc', processInteraction.vcc, 'processId:', processInteraction.processId, 'id:', processInteraction.id, 'section:', processInteraction.section, 'part:', processInteraction.part);
            return Promise.resolve(false);
        });
    }
    DownloadJSON(processInteraction, currentQueueId, executionCode) {
        let path = '';
        let data;
        let silenceInTime;
        let silenceOutTime;
        let cognitiveService;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return cognitive_analysis_implementation_1.GetCognitiveService(processInteraction.vcc, null);
        })
            .then(result => {
            cognitiveService = result;
            let promiseArray = [];
            promiseArray.push(file_manager_1.CreateFolder(config_1.SEMANCTIC_ANALYSIS_FOLDER + '/' + processInteraction.vcc));
            promiseArray.push(file_manager_1.CreateFolder(config_1.SEMANCTIC_ANALYSIS_VIEW_FOLDER + '/' + processInteraction.vcc));
            promiseArray.push(file_manager_1.CreateFolder(config_1.COGNITIVE_ANALYSIS_FOLDER + '/' + processInteraction.vcc));
            return Promise.all(promiseArray);
        })
            .then(_ => {
            let regex = /\.(mp3|wav|ogg|json)$/;
            let jsonFileName = processInteraction.fileName.replace(regex, '.json');
            path = `${config_1.SEMANCTIC_ANALYSIS_FOLDER}/${processInteraction.vcc}/${jsonFileName}`;
            return idatha_service_1.IdathaLogin(processInteraction.vcc);
        })
            .then((userAccessToken) => {
            if (!userAccessToken)
                throw 'NOT_FOUND_ACCESS_TOKEN';
            let regex = /\.(mp3|wav|ogg|json)$/;
            let fileName = processInteraction.fileName.replace(regex, '');
            return this.GetJson(userAccessToken.accessToken, currentQueueId, executionCode, fileName);
        })
            .then((result) => {
            data = result;
            logger.info('[SemanticAnalysisDownloadImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting path:', path);
            if (result) {
                let jsonData = JSON.parse(data);
                let validChar = 0;
                let validWords = 0;
                if (jsonData?.words?.length) {
                    validWords = jsonData.words.length;
                    jsonData.words.forEach(r => {
                        validChar += r.name ? r.name.length + 1 : 0;
                    });
                }
                processInteraction['validChar'] = validChar;
                processInteraction['validWords'] = validWords;
                processInteraction['idathaApi'] = `${config_1.IDATHA_IP}:${config_1.IDATHA_HTTP_PORT}`;
                return file_manager_1.WriteFileContent(path, data, 'utf-8');
            }
            return Promise.resolve(null);
        })
            .then((result) => {
            if (result) {
                let cognitiveAnalysis = new cognitive_analysis_implementation_1.CognitiveAnalysisImplementation();
                let transcription = cognitiveAnalysis.SortJson(JSON.parse(data));
                let phrases = cognitiveAnalysis.SplitTranscriptionContentInPhrases(cognitiveService ? cognitiveService.GetCognitiveEngineCharsPerPhrase() : 100, transcription);
                silenceInTime = transcription.summarySpeechData.dataTotalIn.silence;
                silenceOutTime = transcription.summarySpeechData.dataTotalOut.silence;
                logger.info('[SemanticAnalysisDownloadImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting phrases:', phrases.length);
                return Promise.resolve(phrases);
            }
            return Promise.resolve(null);
        })
            .then((result) => {
            let phrases;
            if (result?.length) {
                processInteraction['charsPerPhrase'] = cognitiveService ? cognitiveService.GetCognitiveEngineCharsPerPhrase() : 0;
                processInteraction['totalPhrases'] = result.length;
                let regex = /\.(mp3|wav|ogg|json)$/;
                let jsonFileName = processInteraction.fileName.replace(regex, '.json');
                let path = `${config_1.COGNITIVE_ANALYSIS_FOLDER}/${processInteraction.vcc}/${jsonFileName}`;
                phrases = { phrases: result };
                let stringPhrases = JSON.stringify(phrases);
                return file_manager_1.WriteFileContent(path, stringPhrases, 'utf-8');
            }
            return Promise.resolve(null);
        })
            .then((result) => {
            if (result) {
                logger.info('[SemanticAnalysisDownloadImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting succes: ready');
                processInteraction['status'] = 6;
                processInteraction['statusDescription'] = processInteraction.interactionType !== 'CALL' ? 'ready' : 'pendingConvertAudio';
                processInteraction['isTranscribed'] = 1;
                processInteraction['retry'] = 0;
                processInteraction['uploadedDate'] = moment().format('x');
                processInteraction['errorFunction'] = null;
                processInteraction['errorMessage'] = null;
                processInteraction['silenceInTime'] = Number(silenceInTime);
                processInteraction['silenceOutTime'] = Number(silenceOutTime);
                processInteraction['silenceTime'] = Number(silenceInTime) + Number(silenceOutTime);
                this.errorData.CleanErrorData();
            }
            else {
                logger.error('[SemanticAnalysisDownloadImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error: JSON_IDATHA_IS_NOT_FOUND transcription content:', data, 'path:', path);
                processInteraction['status'] = -6;
                processInteraction['retry'] = processInteraction['retry'] ? processInteraction['retry'] + 1 : 1;
                processInteraction['uploadedDate'] = moment().format('x');
                processInteraction['errorFunction'] = 'DownloadJSON';
                processInteraction['errorMessage'] = 'JSON_IDATHA_IS_NOT_FOUND';
                this.errorData.SetErrorData('DownloadJSON', processInteraction['errorMessage'], processInteraction['errorMessage']);
            }
            return Promise.resolve(processInteraction);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisDownloadImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            processInteraction['status'] = -6;
            processInteraction['retry'] = processInteraction['retry'] ? processInteraction['retry'] + 1 : 1;
            processInteraction['uploadedDate'] = moment().format('x');
            processInteraction['errorFunction'] = 'DownloadJSON';
            processInteraction['errorMessage'] = errorMessage;
            this.errorData.SetErrorData('DownloadJSON', errorMessage, err);
            return Promise.resolve(processInteraction);
        });
    }
    GetJson(accessToken, currentQueueId, executionCode, fileName) {
        let idatha = new idatha_model_1.IdathaConnection();
        let url = `http://${idatha.ip}:${idatha.httpPort}/api/query/document/${fileName}`;
        let options = {
            uri: url,
            retryProtocol: false,
            timeout: 180000,
            headers: {
                'Authorization': 'Bearer ' + accessToken
            }
        };
        let data = '';
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return request_shared_1.SimpleGetRequest(options);
        })
            .then((response) => {
            try {
                let regex = /\bNaN\b/g;
                data = JSON.stringify(response);
                data = data.replace(regex, 'null');
                this.errorData.CleanErrorData();
                return Promise.resolve(data);
            }
            catch (err) {
                logger.error('[SemanticAnalysisDownloadImplementation::IdathaDocumentWords] executionCode:', executionCode, 'getting queueId:', currentQueueId, err, 'options:', options);
                this.errorData.SetErrorData('IdathaDocumentWords', 'ERROR', err);
                return Promise.resolve(null);
            }
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisDownloadImplementation::IdathaDocumentWords] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage, 'options:', options);
            this.errorData.SetErrorData('IdathaDocumentWords', errorMessage, err);
            return Promise.resolve(null);
        });
    }
}
exports.SemanticAnalysisDownloadImplementation = SemanticAnalysisDownloadImplementation;
