"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SemanticAnalysisUploadImplementation = void 0;
const config_1 = require("../config/config");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const promises_shared_1 = require("../shared/promises.shared");
const log4js = require("log4js");
const moment = require("moment");
const billing_batch_model_1 = require("../models/billing-batch.model");
const idatha_service_1 = require("../services/idatha.service");
const idatha_model_1 = require("../models/idatha.model");
const file_manager_1 = require("../shared/file-manager");
const SSH2Promise = require("ssh2-promise");
const arrays_shared_1 = require("../shared/arrays.shared");
const billing_implementation_1 = require("./billing.implementation");
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
const CURRENT_IMPLEMENTATION = 'SemanticAnalysisUploadImplementation';
class SemanticAnalysisUploadImplementation {
    constructor() {
        this.idathaConnection = new idatha_model_1.IdathaConnection();
        this.sshConnection = null;
        this.sftpConnection = null;
        this.defaultFolders = [];
        this.errorData = new billing_batch_model_1.BatchError();
        this.queueBase = 'SemanticAnalysisUpload';
    }
    Execute(executionCode, rateRecursive) {
        logger.info(`[SemanticAnalysisUploadImplementation::Execute] executionCode: ${executionCode} executing wave #${rateRecursive}`);
        let executeIndex = 1;
        let ratio = config_1.RATIO_SEMANTIC_ANALYSIS_UPLOAD;
        let executeResult = true;
        let currentBlockPromise = null;
        return promises_shared_1.SleepPromise(config_1.SLEEP_SEMANTIC_ANALYSIS)
            .then(_ => {
            currentBlockPromise = 'BLOCK_01_GET_FROM_REDIS';
            return redis_service_1.Redis.GetLengthFromList(this.queueBase);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_02_GET_QUEUE_ITEMS';
            if (!result)
                throw 'SERVER_INFO_REDIS_KEYS_FOR_SEMANTIC_ANALYSIS_UPLOAD_NOT_FOUND';
            return redis_service_1.Redis.LRange(this.queueBase, 0, ratio - 1);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_03_VERIFY_EXTERNAL_CONNECTION';
            ratio = result && result.length <= ratio ? result.length : 0;
            let vccs = arrays_shared_1.GetUniqueFromArray(result.map(r => {
                let items = r.split(':');
                return items[0];
            }));
            return this.VerifyConnection(vccs);
        })
            .then(_ => {
            currentBlockPromise = 'BLOCK_04_SEMANTIC_ANALYSIS_UPLOAD_IMPL';
            let promiseArray = [];
            while (ratio >= executeIndex) {
                promiseArray.push(this.SemanticAnalysisUploadExecute(executionCode));
                executeIndex++;
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then((results) => {
            currentBlockPromise = 'BLOCK_05_GETTING_KEYS_FOR_RECURSIVITY';
            if (results.filter(item => item === false).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange(this.queueBase, 0, ratio - 1);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_06_RELAUNCH_SEMANTIC_ANALYSIS_UPLOAD_IMP';
            if (result?.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1);
            return Promise.resolve(executeResult);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_07_RESOLVING_IMPL';
            if (rateRecursive === config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS_UPLOAD)
                logger.info(`[SemanticAnalysisUploadImplementation::Execute] executionCode: ${executionCode} Finishing execution wave #${rateRecursive}`);
            if (this.sshConnection)
                this.sshConnection.close();
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            if (errorMessage.startsWith('SERVER_INFO_'))
                logger.info('[SemanticAnalysisUploadImplementation::Execute] executionCode:', executionCode, ', Block:', currentBlockPromise, ', getting info:', errorMessage);
            else
                logger.error('[SemanticAnalysisUploadImplementation::Execute] executionCode:', executionCode, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            if (this.sshConnection)
                this.sshConnection.close();
            return Promise.resolve(false);
        });
    }
    SemanticAnalysisUploadExecute(executionCode) {
        let storage = '';
        let currentQueueId = '';
        let processInteraction = null;
        let vcc = '';
        let processId = '';
        let interactionType = '';
        let interactionId = '';
        let interactionSection = 0;
        let interactionPart = 0;
        return redis_service_1.Redis.RPopLPush(this.queueBase, `${this.queueBase}Execute`)
            .then((queueId) => {
            if (queueId?.length > 0) {
                currentQueueId = queueId;
                logger.info('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                let item = queueId.split(':');
                if (item?.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    interactionId = item[3];
                    interactionSection = Number(item[4]);
                    interactionPart = Number(item[5]);
                    storage = vcc + ':' + processId;
                    logger.info('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                    return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
                }
                else {
                    logger.error('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return Promise.resolve(null);
        })
            .then((processInteractionResult) => {
            if (processInteractionResult) {
                processInteraction = processInteractionResult;
                logger.info('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1, status:', processInteraction.statusDescription);
                if (processInteraction.statusDescription === 'pendingAnalysis') {
                    return this.SendInteraction(processInteraction, currentQueueId, executionCode);
                }
            }
            if (!processInteraction && currentQueueId) {
                logger.warn('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                if (!processInteraction) {
                    redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId);
                    throw 'SERVER_ERROR_INTERACTION_DATA_NOT_FOUND_ON_STORAGE';
                }
            }
            return Promise.resolve(null);
        })
            .then((result) => {
            if (result)
                processInteraction = result;
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return Promise.resolve(null);
        })
            .then((result) => {
            let promiseArray = [];
            let billing = new billing_implementation_1.BillingImplementation();
            if (result
                &&
                    processInteraction?.status === 6
                &&
                    processInteraction?.statusDescription === 'pendingDownloadJSON'
                &&
                    processInteraction?.retry < config_1.RETRY_SEMANTIC_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.InsertQueue('SemanticAnalysisDownload', currentQueueId, config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS * config_1.RATIO_SEMANTIC_ANALYSIS * 2));
                promiseArray.push(redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId));
            }
            else if (result
                &&
                    processInteraction?.status === -6
                &&
                    (processInteraction?.statusDescription === 'pendingRemove'
                        ||
                            processInteraction?.retry >= config_1.RETRY_SEMANTIC_ANALYSIS)) {
                let resultMessage = processInteraction?.retry >= config_1.RETRY_SEMANTIC_ANALYSIS ? 'REMOVED_BY_RETRIES' : 'REMOVED_BY_INVALID_CONTENT';
                logger.error('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, resultMessage);
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, resultMessage);
                promiseArray.push(redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId));
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                let interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId));
            }
            else {
                logger.warn('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(_ => {
            let promiseArray = [];
            if (processInteraction
                &&
                    (processInteraction.retry >= config_1.RETRY_SEMANTIC_ANALYSIS
                        ||
                            [
                                'JSON_TRANSCRIPTION_CONTENT_IS_EMPTY',
                                'JSON_TRANSCRIPTION_CONTENT_HAS_NOT_WORD_TYPE_DATA'
                            ].includes(processInteraction.errorMessage))) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, processInteraction.batchTypeId, processInteraction.batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Catching error: ' + errorMessage);
            if (processInteraction)
                logger.error('[SemanticAnalysisUploadImplementation::SemanticAnalysisUploadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage, ' vcc', processInteraction.vcc, 'processId:', processInteraction.processId, 'id:', processInteraction.id, 'section:', processInteraction.section, 'part:', processInteraction.part);
            return Promise.resolve(false);
        });
    }
    SendInteraction(interaction, currentQueueId, executionCode) {
        let path = config_1.TRANSCRIPTION_FOLDER;
        let jsonContent = null;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return this.UpdateJsonMetadata(interaction, currentQueueId, executionCode);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_METADATA_UPDATE_FAILED';
            jsonContent = result;
            if (jsonContent?.results?.length) {
                let wordResults = jsonContent.results.filter(r => r.type === 'word').length;
                if (wordResults)
                    return this.SendJsonFile(interaction, currentQueueId, executionCode);
                else
                    return Promise.resolve('JSON_TRANSCRIPTION_CONTENT_HAS_NOT_WORD_TYPE_DATA');
            }
            else {
                return Promise.resolve('JSON_TRANSCRIPTION_CONTENT_IS_EMPTY');
            }
        })
            .then((result) => {
            if (typeof result === 'boolean' && result === true) {
                interaction['status'] = 6;
                interaction['statusDescription'] = 'pendingDownloadJSON';
                interaction['retry'] = 0;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorMessage'] = null;
                this.errorData.CleanErrorData();
            }
            else {
                logger.error('[SemanticAnalysisUploadImplementation::UploadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'JSON_TRANSCRIPTION_WAS_NOT_TRANSFERRED');
                interaction['status'] = -6;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                if (typeof result === 'string'
                    && [
                        'JSON_TRANSCRIPTION_CONTENT_IS_EMPTY',
                        'JSON_TRANSCRIPTION_CONTENT_HAS_NOT_WORD_TYPE_DATA'
                    ].includes(result)) {
                    logger.error('[SemanticAnalysisUploadImplementation::UploadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, result);
                    interaction['statusDescription'] = 'pendingRemove';
                    interaction['errorMessage'] = result;
                }
                else {
                    interaction['errorMessage'] = 'JSON_TRANSCRIPTION_WAS_NOT_TRANSFERRED';
                }
                this.errorData.SetErrorData('UploadJSON', interaction['errorMessage'], interaction['errorMessage']);
            }
            logger.info('[SemanticAnalysisUploadImplementation::UploadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, `Resolving for ${path}`);
            return Promise.resolve(interaction);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisUploadImplementation::UploadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, errorMessage);
            interaction['status'] = -6;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorMessage'] = errorMessage;
            this.errorData.SetErrorData('UploadJSON', errorMessage, err);
            return Promise.resolve(interaction);
        });
    }
    UpdateJsonMetadata(interaction, currentQueueId, executionCode) {
        let path = config_1.TRANSCRIPTION_FOLDER;
        let jsonContent = null;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return idatha_service_1.IdathaLogin(interaction.vcc);
        })
            .then((result) => {
            if (!result || !result.accessToken)
                throw 'SERVER_ERROR_CANNOT_GET_USER_ACCESS_TOKEN';
            let regex = /\.(mp3|wav|ogg)$/;
            let jsonFileName = interaction.fileName;
            if (!jsonFileName) {
                jsonFileName = `${interaction.id}_${interaction.section}_${interaction.part}.${interaction.format || 'wav'}`;
                interaction.fileName = jsonFileName;
            }
            jsonFileName = interaction.fileName.replace(regex, '.json');
            path += `/${interaction.vcc}/${jsonFileName}`;
            logger.info('[SemanticAnalysisUploadImplementation::UpdateJsonMetadata] executionCode:', executionCode, 'getting queueId:', currentQueueId, `Reading file ${path}`);
            return file_manager_1.ReadFileContent(path, 'utf-8');
        })
            .then((result) => {
            try {
                jsonContent = typeof result === 'string' ? JSON.parse(result) : result;
                jsonContent.filterdata = {
                    start_date: new Date(interaction.startDate),
                    end_date: new Date(interaction.endDate),
                    channel: interaction.interactionType,
                    campaign: interaction.campaign,
                    disposition: interaction.dispositionCode,
                    agent: interaction.agent,
                    origin: interaction.origin
                };
            }
            catch (ex) {
                jsonContent = '';
                logger.error('[SemanticAnalysisUploadImplementation::UpdateJsonMetadata] executionCode:', executionCode, 'getting queueId:', currentQueueId, `error on parsing json:`, ex);
                throw 'ERROR_FILE_JSON';
            }
            logger.info('[SemanticAnalysisUploadImplementation::UpdateJsonMetadata] executionCode:', executionCode, 'getting queueId:', currentQueueId, `Writing file ${path}`);
            return file_manager_1.WriteFileContent(path, JSON.stringify(jsonContent), 'utf-8', true);
        })
            .then(_ => {
            return Promise.resolve(jsonContent);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisUploadImplementation::UpdateJsonMetadata] executionCode:', executionCode, 'getting queueId:', currentQueueId, errorMessage);
            interaction['status'] = -6;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorMessage'] = errorMessage;
            this.errorData.SetErrorData('UpdateJsonMetadata', errorMessage, err);
            return Promise.resolve(null);
        });
    }
    SendJsonFile(processInteraction, currentQueueId, executionCode) {
        let regex = /\.(mp3|wav|ogg)$/;
        let src = `${config_1.TRANSCRIPTION_FOLDER}/${processInteraction.vcc.toLowerCase()}/${processInteraction.fileName.replace(regex, '.json')}`;
        let target = `/home/${this.idathaConnection.user}/in_concert_speech_analytics/input/${processInteraction.vcc.toLowerCase()}/default/`;
        let destination = `${target}${processInteraction.fileName.replace(regex, '.json')}`;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (this.defaultFolders.includes(processInteraction.vcc))
                return this.UploadJson(src, destination);
            return Promise.resolve(false);
        })
            .then((result) => {
            if (result)
                this.errorData.CleanErrorData();
            return Promise.resolve(result);
        })
            .catch(err => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisImplementation::IdathaSendJSONFiles] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            this.errorData.SetErrorData('IdathaSendJSONFiles', errorMessage, err);
            return Promise.resolve(false);
        });
    }
    UploadJson(src, destination) {
        this.sftpConnection ?? (this.sftpConnection = this.sshConnection.sftp());
        logger.info(`[SemanticAnalysisUploadImplementation::UploadJson] Uploading from: "${src}"`);
        return promises_shared_1.DummyPromise()
            .then(_ => this.sftpConnection.fastPut(src, destination))
            .then(_ => {
            logger.info(`[SemanticAnalysisUploadImplementation::UploadJson] Uploaded corretly: "${src}"`);
            return Promise.resolve(true);
        })
            .catch(err => {
            logger.error('[SemanticAnalysisUploadImplementation::UploadJson]', err);
            return Promise.reject(err);
        });
    }
    VerifyConnection(vccs) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return this.ConnectToSsh();
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_CONNECT_TO_IDATHA_SSH';
            let promiseArray = [];
            vccs.forEach(vcc => {
                promiseArray.push(this.CreateDefaultDirIfNotExists(vcc));
            });
            return Promise.all(promiseArray);
        })
            .then(results => {
            if (results.includes(false))
                logger.error('[SemanticAnalysisUploadImplementation::VerifyConnection] Error creating a default folder');
            return Promise.resolve(true);
        })
            .catch(error => {
            logger.error('[SemanticAnalysisUploadImplementation::VerifyConnection] Getting error:', error);
            return Promise.resolve(false);
        });
    }
    ConnectToSsh() {
        let sshConfig = {
            host: this.idathaConnection.ip,
            port: this.idathaConnection.sshPort,
            username: this.idathaConnection.user,
            password: this.idathaConnection.password,
            connTimeout: 30000,
            pasvTimeout: 30000
        };
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (!this.sshConnection) {
                let ssh2 = new SSH2Promise(sshConfig);
                this.sshConnection = ssh2;
            }
            return Promise.resolve(true);
        })
            .catch(error => {
            logger.error('[SemanticAnalysisUploadImplementation::ConnectToSsh] Getting error:', error);
            return Promise.resolve(false);
        });
    }
    CreateDefaultDirIfNotExists(vcc) {
        let path = `/home/${this.idathaConnection.user}/in_concert_speech_analytics/input/${vcc.toLowerCase()}/default/`;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (!this.defaultFolders.includes(vcc))
                return this.sshConnection.exec(`mkdir -p ${path}`);
            return Promise.resolve(true);
        })
            .then(_ => {
            this.defaultFolders.push(vcc);
            return Promise.resolve(true);
        })
            .catch((err) => {
            logger.error('[SemanticAnalysisUploadImplementation::CreateDefaultDirIfNotExists]', err);
            return Promise.resolve(false);
        });
    }
}
exports.SemanticAnalysisUploadImplementation = SemanticAnalysisUploadImplementation;
