"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetCustomField = exports.CustomFieldService = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const custom_field_model_1 = require("../models/custom-field.model");
let logger = log4js.getLogger('ServerScripts');
function CustomFieldService(app) {
    app.get('/api/customfield/get_customfields/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info('[CustomFieldService::get_customfields] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let search_criteria = req.params.search_criteria || '';
        let search_expression = req.params.search_expression || '';
        let disabled = req.query.disabled === 'true' || false;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetCustomFields(vcc, search_criteria, search_expression, disabled);
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[CustomFieldService::get_customfields] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/customfield/get_customfield/:customfield_id', (req, res, next) => {
        logger.info('[CustomFieldService::get_customfields] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let customField = {};
        let customFieldResults = [];
        let customFieldId = req.params.customfield_id;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!customFieldId)
                throw 'SERVER_ERROR_MISSING_PARAMETER';
            return GetCustomField(vcc, customFieldId);
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[CustomFieldService::get_customfields] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/customfield/create_customfield', (req, res, next) => {
        logger.info('[CustomFieldService::save_customfield] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let customField = (req.body || {});
        let isCustomFieldCreated = false;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetCustomField(vcc, customField.id);
        })
            .then(result => {
            if (result)
                throw 'SERVER_ERROR_CUSTOM_FIELD_ID_ALREADY_EXISTS';
            return mssql_service_1.MSSql.SaveCustomField(vcc, customField.id, customField.description, customField.type, customField.label, userId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CUSTOM_FIELD_SAVE_FAILED';
            isCustomFieldCreated = true;
            let promiseArray = [];
            customField.results.map(r => {
                let result = new custom_field_model_1.CustomFieldResult();
                result.id = r.id;
                result.order = r.order;
                result.description = r.description;
                result.customFieldId = customField.id;
                result.createdByUserId = userId;
                promiseArray.push(mssql_service_1.MSSql.SaveCustomFieldResult(vcc, result.id, result.description, result.order, result.customFieldId, result.createdByUserId));
            });
            return Promise.all(promiseArray);
        })
            .then(result => {
            if (result.includes(false))
                throw 'SERVER_ERROR_CUSTOM_FIELD_RESULT_SAVE_FAILED';
            if (res.headersSent)
                return;
            logger.info('[CustomFieldService::create_customfield] The CustomField save proccess has been finished.');
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_CUSTOM_FIELD_RESULT_SAVE_FAILED'), data: true });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[CustomFieldService::create_customfield] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
            if (isCustomFieldCreated)
                CustomFieldDelete(vcc, customField);
        });
    });
    app.post('/api/customfield/save_customfield', (req, res, next) => {
        logger.info('[CustomFieldService::save_customfield] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let customField = (req.body || {});
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetCustomField(vcc, customField.id);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CUSTOM_FIELD_NOT_FOUND';
            return mssql_service_1.MSSql.SaveCustomField(vcc, customField.id, customField.description, customField.type, customField.label, userId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CUSTOM_FIELD_SAVE_FAILED';
            return mssql_service_1.MSSql.DeleteCustomFieldResults(vcc, customField.id);
        })
            .then(result => {
            let promiseArray = [];
            customField.results.map(r => {
                let result = new custom_field_model_1.CustomFieldResult();
                result.id = r.id;
                result.order = r.order;
                result.description = r.description;
                result.customFieldId = customField.id;
                result.lastModifiedByUserId = userId;
                promiseArray.push(mssql_service_1.MSSql.SaveCustomFieldResult(vcc, result.id, result.description, result.order, result.customFieldId, result.lastModifiedByUserId));
            });
            return Promise.all(promiseArray);
        })
            .then(result => {
            if (result.includes(false))
                throw 'SERVER_ERROR_CUSTOM_FIELD_RESULT_SAVE_FAILED';
            if (res.headersSent)
                return;
            logger.info('[CustomFieldService::save_customfield] The CustomField save proccess has been finished.');
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_CUSTOM_FIELD_RESULT_SAVE_FAILED'), data: true });
        })
            .catch(err => {
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            let clientMsg = (typeof err === 'string' ? err : (errorMsg || 'CANNOT_PROCESS_REQUEST'));
            logger.error('[CustomFieldService::save_customfield] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/customfield/delete_customfields', (req, res, next) => {
        logger.info('[CustomFieldService::delete_customfield] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let customFields = (req.body || []);
        let ids = customFields.map(item => item.id);
        logger.info(`[CustomFieldService::delete_customfields] Deleting customFields ${ids.join(',')} by the user ${res.locals.userId}`);
        let response;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetCustomFieldsUsage(vcc, ids);
        })
            .then(resultNoDeleted => {
            let deleteData = ids;
            let noDelete = [];
            if (resultNoDeleted.length > 0) {
                deleteData = [];
                customFields.forEach(item => {
                    if (!resultNoDeleted.find(noDeleted => noDeleted.customFieldId === item.id))
                        deleteData.push(item.id);
                    else
                        noDelete.push(item.id);
                });
            }
            return Promise.resolve({ noDelete, deleteData });
        }).then(resultDelete => {
            response = resultDelete;
            if (resultDelete.deleteData.length) {
                return mssql_service_1.MSSql.DeleteCustomFields(vcc, resultDelete.deleteData);
            }
            return Promise.resolve(true);
        })
            .then(result => {
            if (res.headersSent)
                return;
            logger.info('[CustomFieldService::delete_customfield] The CustomField delete proccess has been finished.');
            res.json({ status: result, description: 'SERVER_SUCCESS_MESSAGE', data: response });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[CustomFieldService::delete_customfield] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/customfield/delete_customfield', (req, res, next) => {
        logger.info('[CustomFieldService::delete_customfield] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let customField = (req.body || {});
        logger.info(`[CustomFieldService::delete_customfield] Deleting customField ${customField} by the user ${res.locals.userId}`);
        return promises_shared_1.DummyPromise()
            .then(result => {
            return CustomFieldDelete(vcc, customField);
        })
            .then(result => {
            if (result && result.status === false)
                throw result.error;
            if (res.headersSent)
                return;
            logger.info('[CustomFieldService::delete_customfield] The CustomField delete proccess has been finished.');
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: true });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[CustomFieldService::delete_customfield] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    function CustomFieldDelete(vcc, customField) {
        let genericResult = {};
        return promises_shared_1.DummyPromise()
            .then(result => {
            logger.error('[CustomFieldService::CustomFieldDelete] Result: ' + result);
            return mssql_service_1.MSSql.DeleteCustomField(vcc, customField.id);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CUSTOM_FIELD_DELETE_FAILED';
            genericResult.status = true;
            genericResult.description = 'OK';
            return Promise.resolve(genericResult);
        })
            .catch(err => {
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_CUSTOM_FIELD_DELETE_FAILED');
            logger.error('[CustomFieldService::CustomFieldDelete] Error: ' + errorMsg);
            genericResult.status = false;
            genericResult.description = 'SERVER_ERROR_UNKNOWN_MESSAGE';
            genericResult.error = errorMsg;
            return Promise.resolve(genericResult);
        });
    }
    ;
}
exports.CustomFieldService = CustomFieldService;
function GetCustomField(vcc, customFieldId, evaluationPlanId = null) {
    let customField = {};
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return mssql_service_1.MSSql.GetCustomField(vcc, customFieldId, evaluationPlanId);
    })
        .then(result => {
        customField = result;
        if (customField)
            return mssql_service_1.MSSql.GetCustomFieldResults(vcc, customFieldId, evaluationPlanId);
        else
            return Promise.resolve(true);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_CUSTOM_FIELD_RESULTS_NOT_FOUND';
        if (customField)
            customField.results = result;
        return Promise.resolve(customField);
    })
        .catch(err => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[CustomFieldService::GetCustomField] Error: ' + errorMsg);
        return Promise.reject(false);
    });
}
exports.GetCustomField = GetCustomField;
