"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UpdateClientPublicKey = exports.DecryptClientString = exports.EncryptClientString = exports.GenerateKeyPairSync = void 0;
const crypto = require("crypto");
const log4js = require("log4js");
const path = require("path");
const uuid = require("uuid");
const server_config_1 = require("../config/server.config");
const promises_shared_1 = require("../shared/promises.shared");
const file_manager_1 = require("../shared/file-manager");
let replace = require('replace');
let logger = log4js.getLogger('ServerScripts');
function GenerateKeyPairSync() {
    const keyPairSync = [];
    const { publicKey, privateKey } = crypto.generateKeyPairSync('rsa', {
        modulusLength: 2048,
        publicKeyEncoding: {
            type: 'pkcs1',
            format: 'pem'
        },
        privateKeyEncoding: {
            type: 'pkcs1',
            format: 'pem',
        }
    });
    if (publicKey && privateKey) {
        keyPairSync.push(publicKey);
        keyPairSync.push(privateKey);
    }
    return keyPairSync;
}
exports.GenerateKeyPairSync = GenerateKeyPairSync;
function EncryptClientString(str) {
    let encryptedStr = '';
    if (!str)
        return encryptedStr;
    try {
        encryptedStr = crypto.publicEncrypt({
            key: server_config_1.CLIENT_DATA_PUBLIC_KEY,
            padding: crypto.constants.RSA_PKCS1_PADDING,
        }, Buffer.from(str)).toString('base64') || '';
    }
    catch (err) {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || '');
        logger.error('Error al encriptar datos de cliente: ' + errorMsg);
        encryptedStr = '';
    }
    return encryptedStr;
}
exports.EncryptClientString = EncryptClientString;
function DecryptClientString(str) {
    let decryptedStr = '';
    if (!str)
        return decryptedStr;
    try {
        decryptedStr = crypto.privateDecrypt({
            key: server_config_1.CLIENT_DATA_PRIVATE_KEY,
            padding: crypto.constants.RSA_PKCS1_PADDING,
        }, Buffer.from(str, 'base64')).toString('utf8') || '';
    }
    catch (err) {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || '');
        logger.error('Error al decriptar datos de cliente: ' + errorMsg);
        decryptedStr = '';
    }
    return decryptedStr;
}
exports.DecryptClientString = DecryptClientString;
function UpdateClientPublicKey(indexPath, indexFormPath) {
    let lockToken = uuid.v4();
    let lockFilename = path.resolve(__dirname, '..', '..', '__UpdateClientPublicKey.lock');
    let serverConfigFilename = path.resolve(__dirname, '..', 'config', 'server.config.js');
    let serverBackupFilename = path.resolve(__dirname, '..', 'config', '_server.config.js');
    let mustUpdatePublicKey = true;
    let mustDeleteLockFile = false;
    promises_shared_1.DummyPromise()
        .then(result => {
        return file_manager_1.WriteFileContent(lockFilename, lockToken, 'utf8', true);
    }).then(resultWrite => {
        return promises_shared_1.PromiseWait(2000, true);
    }).then(resultTimeout => {
        return file_manager_1.ReadFileContent(lockFilename, 'utf8', true);
    }).then(resultRead => {
        if (resultRead !== lockToken)
            mustUpdatePublicKey = false;
        else
            mustDeleteLockFile = true;
        if (!mustUpdatePublicKey)
            return null;
        return Promise.all([
            file_manager_1.FileProperty(serverConfigFilename),
            file_manager_1.FileProperty(serverBackupFilename),
            file_manager_1.ReadFileContent(serverConfigFilename, 'utf8', true),
            file_manager_1.ReadFileContent(serverBackupFilename, 'utf8', true),
        ]);
    }).then(resultFilesContent => {
        if (!mustUpdatePublicKey)
            return null;
        let serverConfigFileStats = resultFilesContent[0] || {};
        let serverBackupFileStats = resultFilesContent[1] || {};
        let serverConfigFileContent = resultFilesContent[2] || '';
        let serverBackupFileContent = resultFilesContent[3] || '';
        if (serverConfigFileContent && serverConfigFileContent === serverBackupFileContent
            && serverBackupFileStats.updateDate && serverConfigFileStats.updateDate
            && serverBackupFileStats.updateDate.isAfter(serverConfigFileStats.updateDate)) {
            logger.info('La configuración de clave pública no cambió, por lo que no hace falta actualizar archivos de cliente');
            mustUpdatePublicKey = false;
            return true;
        }
        return file_manager_1.WriteFileContent(serverBackupFilename, serverConfigFileContent, 'utf8', true);
    }).then(resultWrite => {
        if (!mustUpdatePublicKey)
            return null;
        logger.info('Actualizando configuración de clave pública en los archivos de cliente');
        replace({
            regex: /window\.CLIENT_DATA_PUBLIC_KEY\s*=\s*[^;]+/,
            replacement: 'window.CLIENT_DATA_PUBLIC_KEY = `' + server_config_1.CLIENT_DATA_PUBLIC_KEY + '`',
            paths: [indexPath],
            recursive: false,
            silent: true,
        });
        replace({
            regex: /window\.CLIENT_DATA_PUBLIC_KEY\s*=\s*[^;]+/,
            replacement: 'window.CLIENT_DATA_PUBLIC_KEY = `' + server_config_1.CLIENT_DATA_PUBLIC_KEY + '`',
            paths: [indexFormPath],
            recursive: false,
            silent: true,
        });
        return true;
    }).then(resultWrite => {
        if (mustDeleteLockFile)
            return file_manager_1.DeleteFile(lockFilename);
        return true;
    }).then(resultDelete => {
        return true;
    }).catch(err => {
        let errMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('Error al actualizar clave pública en archivos de cliente: ' + errMsg);
        return null;
    });
}
exports.UpdateClientPublicKey = UpdateClientPublicKey;
