"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.KillTask = exports.FindProcess = void 0;
const child_process = require("child_process");
const path = require("path");
const moment = require("moment");
const UNIT_MB = 1024 * 1024;
function FindProcess(value) {
    let platform = process.platform;
    return new Promise((resolve, reject) => {
        if (!(platform in finders)) {
            reject(new Error(`platform ${platform} is unsupported`));
        }
        let find = finders[platform];
        if (typeof find === 'string') {
            find = finders[find];
        }
        find(value).then(resolve, reject);
    });
}
exports.FindProcess = FindProcess;
function KillTask(pid) {
    let platform = process.platform;
    return new Promise((resolve, reject) => {
        if (!(platform in killers)) {
            reject(new Error(`platform ${platform} is unsupported`));
        }
        let find = killers[platform];
        if (typeof find === 'string') {
            find = killers[find];
        }
        find(pid).then(resolve, reject);
    });
}
exports.KillTask = KillTask;
function ExecCommand(cmd, callback) {
    child_process.exec(cmd, { maxBuffer: 2 * UNIT_MB }, callback);
}
function StripLine(text, num) {
    let idx = 0;
    while (num-- > 0) {
        let nIdx = text.indexOf('\n', idx);
        if (nIdx >= 0) {
            idx = nIdx + 1;
        }
    }
    return idx > 0 ? text.substring(idx) : text;
}
function MatchName(text, name) {
    if (!name) {
        return true;
    }
    return text.match(name);
}
function SplitCustom(line, max) {
    let cols = line.trim().split(/\s+/);
    if (cols.length > max) {
        cols[max - 1] = cols.slice(max - 1).join(' ');
    }
    return cols;
}
function ExtractColumns(text, idxes, max) {
    let lines = text.split(/(\r\n|\n|\r)/);
    let columns = [];
    if (!max) {
        max = Math.max.apply(null, idxes) + 1;
    }
    lines.forEach((line) => {
        let cols = SplitCustom(line, max);
        let column = [];
        idxes.forEach((idx) => {
            column.push(cols[idx] || '');
        });
        columns.push(column);
    });
    return columns;
}
function ParseTable(data) {
    let lines = data.split(/(\r\n|\n|\r)/).filter((line) => {
        return line.trim().length > 0;
    });
    let matches = lines.shift().trim().match(/(\w+\s*)/g);
    if (!matches) {
        return [];
    }
    let ranges = [];
    let headers = matches.map((col, i) => {
        let range = [];
        if (i === 0) {
            range[0] = 0;
            range[1] = col.length;
        }
        else {
            range[0] = ranges[i - 1][1];
            range[1] = range[0] + col.length;
        }
        ranges.push(range);
        return col.trim();
    });
    ranges[ranges.length - 1][1] = Infinity;
    return lines.map((line) => {
        let row = {};
        ranges.forEach((r, i) => {
            let key = headers[i];
            let value = line.substring(r[0], r[1]).trim();
            row[key] = value;
        });
        return row;
    });
}
const finders = {
    darwin(name) {
        return new Promise((resolve, reject) => {
            let cmd = `ps ax -ww -o pid,ppid,uid,gid,etime,args`;
            ExecCommand(cmd, function (err, stdout, stderr) {
                if (err) {
                    reject(err);
                }
                else {
                    err = stderr.toString().trim();
                    if (err) {
                        reject(err);
                        return;
                    }
                    let data = StripLine(stdout.toString(), 1);
                    let columns = ExtractColumns(data, [0, 1, 2, 3, 4, 5], 6).filter((column) => {
                        if (column[0]) {
                            return MatchName(column[5], name);
                        }
                        else {
                            return false;
                        }
                    });
                    let list = columns.map((column) => {
                        let cmd = String(column[5]).split(' ', 1)[0];
                        let dayArray = column[4].split('-');
                        let timeArray = (dayArray && dayArray.length === 2) ? dayArray[1].split(':') : dayArray[0].split(':');
                        let timeSecond = (timeArray && timeArray.length === 3) ? parseInt(timeArray[0]) * 60 * 60 + parseInt(timeArray[1]) * 60 + parseInt(timeArray[2]) : parseInt(timeArray[0]) * 60 + parseInt(timeArray[1]);
                        let totalSecond = (dayArray && dayArray.length === 2) ? parseInt(dayArray[0]) * 24 * 60 * 60 + timeSecond : timeSecond;
                        return {
                            pid: column[0],
                            ppid: column[1],
                            uid: column[2],
                            gid: column[3],
                            name: path.basename(cmd),
                            cmd: column[5],
                            day: column[4],
                            time: totalSecond
                        };
                    });
                    resolve(list);
                }
            });
        });
    },
    linux: 'darwin',
    sunos: 'darwin',
    freebsd: 'darwin',
    win32(name) {
        return new Promise((resolve, reject) => {
            let cmd = 'WMIC path win32_process get Name,Processid,ParentProcessId,Commandline,CreationDate';
            ExecCommand(cmd, function (err, stdout, stderr) {
                if (err) {
                    reject(err);
                }
                else {
                    err = stderr.toString().trim();
                    if (err) {
                        reject(err);
                        return;
                    }
                    let list = ParseTable(stdout.toString()).filter((row) => {
                        return MatchName(row.CommandLine, name);
                    }).map((row) => {
                        let valueNull = null;
                        let dayOriginal = row.CreationDate;
                        let day = dayOriginal.substr(0, 4) + '-' + dayOriginal.substr(4, 2) + '-' + dayOriginal.substr(6, 2) + ' ' + dayOriginal.substr(8, 2) + ':' + dayOriginal.substr(10, 2) + ':' + dayOriginal.substr(12, 2);
                        let dayMoment = moment(day);
                        return {
                            pid: row.ProcessId,
                            ppid: row.ParentProcessId,
                            uid: valueNull,
                            gid: valueNull,
                            name: row.Name,
                            cmd: row.CommandLine,
                            date: day,
                            time: moment().diff(dayMoment, 'seconds')
                        };
                    });
                    resolve(list);
                }
            });
        });
    }
};
const killers = {
    darwin(pid) {
        return new Promise((resolve, reject) => {
            let cmd = `kill ${pid}`;
            ExecCommand(cmd, function (err, stdout, stderr) {
                if (err) {
                    reject(err);
                }
                else {
                    err = stderr.toString().trim();
                    if (err) {
                        reject(err);
                        return;
                    }
                    resolve(true);
                }
            });
        });
    },
    linux: 'darwin',
    sunos: 'darwin',
    freebsd: 'darwin',
    win32(pid) {
        return new Promise((resolve, reject) => {
            let cmd = `Taskkill /PID ${pid} /F`;
            ExecCommand(cmd, function (err, stdout, stderr) {
                if (err) {
                    reject(err);
                }
                else {
                    err = stderr.toString().trim();
                    if (err) {
                        reject(err);
                        return;
                    }
                    resolve(true);
                }
            });
        });
    }
};
