/*jshint bitwise:true, browser:true, curly:true, eqeqeq:true, noarg:true, noempty:true, strict:true, undef:true, unused:true */
/*jshint quotmark:false, esversion: 9*/
/*global _, SIPml, inConcert, console, $*/

this.inConcert = this.inConcert || {};

(function () {
	"use strict";

	/*!
	 * Possible phone states
	 */
	var PhoneStates = {
		IDLE       : "idle",
		CONNECTING : "connecting",
		CONNECTED  : "connected",
		OUTBOUND   : "outbound",
		INBOUND    : "inbound",
		ACTIVE     : "active"
	};

	/*!
	 * A wrapper around SIPml
	 *
	 * Events:
	 * \li rejected(reason)
	 * \li warning(message)
	 * \li error(message)
	 * \li invited(callerid)
	 * \li connected()
	 * \li connecting()
	 * \li dialing()
	 * \li connected() -> after connecting()
	 * \li accepted()  -> after dialing()
	 * \li cancelled()
	 * \li byed()
	 * \li ringing()
	 */
	function SipPhone(params) {
		// Initial setup
		_.extend(this, {
			// public, handles the state machine
			currentState    : PhoneStates.IDLE,
			muted: false,
			switched: false,

			m_initialized   : false,
			m_error         : null,
			m_sipStack      : null,
			m_regSession    : null,
			m_callSession   : null,
			m_data          : null,
			m_defaultHeaders: [
				{ name: 'User-Agent'  , value: 'inConcert Builtin Phone/sipML5-v1.4.217' },
				{ name: 'Organization', value: 'inConcert' }
			],
			m_hasPermission: true,
			m_callConfig   : undefined,
			m_localVideo   : undefined,
			m_remoteVideo  : undefined,
			m_remoteAudio  : undefined
		}, params);

		this.logger = this.logger || {
			debug : _.bind(console.debug, console),
			info  : _.bind(console.info, console),
			warn  : _.bind(console.warn, console),
			error : _.bind(console.error, console)
		};
		SIPml.setDebugLevel("warn");
	}

	/*!
	 * Connects and registers with Asterisk (aka the SIP server)
	 */
	SipPhone.prototype.connect = function(data, callback = $.noop) {
		this.logger.debug("SIPPhone: Connect");
		this.m_data = data;
		if (this.currentState !== PhoneStates.IDLE) {
			return this.__invalidState("connect");
		}
		this.__setState(PhoneStates.CONNECTING);
		this.__connect(data, callback);
		this.logger.debug("SIPPhone: Connected");
	};

	/*!
	 * Disconnects from the SIP server, closes all sessions
	 */
	SipPhone.prototype.disconnect = function(callback = $.noop) {
		this.logger.debug("SIPPhone: Disconnect");
		this.__setState(PhoneStates.IDLE);
		if (this.m_sipStack) {
			this.m_sipStack.stop(); // shutdown all sessions
			this.m_sipStack = null;
		}
		callback(true);
		this.logger.debug("SIPPhone: Disconnected");
	};

	/*!
	 * Makes a call
	 */
	SipPhone.prototype.invite = function(campaign, prefix, dest, contact, callback = $.noop) {
		if (this.currentState !== PhoneStates.CONNECTED) {
			this.__invalidState("invite");
			return callback(false, "Cannot call while " + this.currentState);
		}

		var number = prefix + dest;
		this.__setState(PhoneStates.OUTBOUND);
		this.m_callSession = this.m_sipStack.newSession('call-audio', this.__getCallConfig());
		if (this.m_callSession.call(number) === 0) {
			callback(true);
		}
		else {
			this.__setState(PhoneStates.CONNECTED);
			callback(false, "Failed to make call");
		}
	};

	/*!
	 * Accepts an incoming call
	 */
	SipPhone.prototype.accept = function(callback = $.noop) {
		if (this.currentState !== PhoneStates.INBOUND) {
			this.__invalidState("accept");
			return callback(false, "Can not accept on: " + this.currentState);
		}
		if (this.m_hasPermission === false) {
			this.__permissionPending('accept');
			return callback(false, "Has not permissions");
		}
		this.m_callSession.accept(this.__getCallConfig());
		callback(true);
		return true;
	};

	/*!
	 * Rejects an incoming call
	 */
	SipPhone.prototype.reject = function(reason, callback = $.noop) {
		if (this.currentState !== PhoneStates.INBOUND) {
			this.__invalidState("reject");
			return callback(false, "Can not reject on: " + this.currentState);
		}
		this.m_callSession.reject();
		callback(true);
	};

	/*!
	 * Hangs-up the active call
	 */
	SipPhone.prototype.bye = function(callback = $.noop) {
		if (!_.contains([PhoneStates.ACTIVE, PhoneStates.OUTBOUND], this.currentState)) {
			this.__invalidState("hangup");
			return callback(false, "Can not bye on: " + this.currentState);
		}
		this.m_callSession.hangup();
		callback(true);
	};

	/*!
	 * Sends a DTMF digit
	 */
	SipPhone.prototype.sendDigit = function(digit, callback = $.noop) {
		if (this.currentState !== PhoneStates.ACTIVE) {
			this.__invalidState("dtmf");
			return callback(false, "Can not send dtmf on: " + this.currentState);
		}
		if ( "1234567890ABCD*#".indexOf(digit) >= 0 ) {
			this.m_callSession.dtmf(digit);
			callback(true);
		}
		else {
			callback(false, "Invalid dtmf");
		}
	};

	/*!
	 * Puts the current call on hold
	 */
	SipPhone.prototype.hold = function(callback = $.noop) {
		if (this.currentState !== PhoneStates.ACTIVE) {
			this.__invalidState("hold");
			return callback(false, "Can not hold on: " + this.currentState);
		}
		this.m_callSession.hold();
		callback(true);
	};

	/*!
	 * Resumes the current call
	 */
	SipPhone.prototype.resume = function(callback = $.noop) {
		if (this.currentState !== PhoneStates.ACTIVE) {
			this.__invalidState("resume");
			return callback(false, "Can not resume on: " + this.currentState);
		}
		this.m_callSession.resume();
		callback(true);
	};

	/*!
	 * Puts the current call on mute
	 */
	SipPhone.prototype.mute = function(callback = $.noop) {
		if (this.currentState !== PhoneStates.ACTIVE) {
			this.__invalidState("mute");
			return callback(false, "Can not mute on: " + this.currentState);
		}
		this.m_callSession.mute("audio", true);
		this.muted = true;
		callback(true);
	};

	/*!
	 * Removes the current call on mute
	 */
	SipPhone.prototype.unmute = function(callback = $.noop) {
		if (this.currentState !== PhoneStates.ACTIVE) {
			this.__invalidState("unmute");
			return callback(false, "Can not unmute on: " + this.currentState);
		}
		this.m_callSession.mute("audio", false);
		this.muted = false;
		callback(true);
	};

	/*!
	 * Switch the current call conference to consultation_call 
	 */
	SipPhone.prototype._switch = function(callback = $.noop) {
		if (this.currentState !== PhoneStates.ACTIVE) {
			this.__invalidState("switch");
			return callback(false, "Can not switch on: " + this.currentState);
		}
		//this.m_callSession.mute("audio", true);
		this.switched = true;
		callback(true);
	};

	/*!
	 * Switch the conference consultation_call to call 
	 */
	SipPhone.prototype.unswitch = function(callback = $.noop) {
		if (this.currentState !== PhoneStates.ACTIVE) {
			this.__invalidState("unswitch");
			return callback(false, "Can not unswitch on: " + this.currentState);
		}
		//this.m_callSession.mute("audio", false);
		this.switched = false;
		callback(true);
	};

	SipPhone.prototype.reconnect = function() {
		var self = this;
		setTimeout(function() {
			self.logger.debug("SIPPhone: Reconnecting");
			self.connect(self.m_data);
		}, 1000);
	};

	/*****************************/
	/**    PRIVATE PROPERTY     **/
	/**                         **/
	/**     NO TRESPASSING      **/
	/**                         **/
	/**        KEEP OUT         **/
	/**                         **/
	/**        VIOLATORS        **/
	/**         WILL BE         **/
	/**        PROSECUTED       **/
	/*****************************/

	SipPhone.prototype.__invalidState = function(action) {
		var message = "Cannot " + action + " while " + this.currentState;
		this.logger.warn("SIPPhone: " + message);
		return false;
	};

	SipPhone.prototype.__permissionPending = function(/*action*/) {
		var message = "You must gran permission to use microphone";
		this.logger.warn("SIPPhone: " + message);
		return false;
	};

	SipPhone.prototype.__emitError = function(message) {
		this.logger.error("SIPPhone: " + message);
	};

	SipPhone.prototype.__getCallConfig = function() {
		this.logger.debug("SIPPhone: GetCallConfig");
		this.m_remoteAudio = this.m_remoteAudio || document.getElementById("telephony-remote-audio");
		this.m_callConfig = this.m_callConfig || {
			audio_remote: this.m_remoteAudio,
			video_local : undefined, //this.m_localVideo,
			video_remote: undefined, //this.m_remoteVideo,
			bandwidth: {
				audio: undefined,
				video: undefined
			},
			video_size: {
				minWidth: undefined, minHeight: undefined,
				maxWidth: undefined, maxHeight: undefined
			},
			events_listener: {
				events: '*',
				listener: _.bind(this.__sessionEventListener, this)
			},
			sip_caps: [
				{ name: '+g.oma.sip-im'},
				{ name: '+sip.ice'     },
				{ name: 'language', value: '\"en,fr\"' }
			]
        	};
		this.logger.debug("SIPPhone: CallConfig = " + JSON.stringify(this.m_callConfig));
        	return this.m_callConfig;
	};

	SipPhone.prototype.__init = function(callback) {
		this.logger.debug("SIPPhone: Init");
		if (this.m_initialized) {
			this.logger.debug("SIPPhone: Init - already initialized");
			return callback();
		}

		var self = this;
		SIPml.init(function() {
			self.logger.debug("SIPPhone: Init - initialized");
			self.m_initialized = true;
			return callback();
		}, function(e) {
			self.logger.debug("SIPPhone: Init - error initializing: " + e);
			callback(e);
		});
	};

	/*!
	 * Initializes the SIP Stack, connects to the server but does
	 * not send anything
	 */
	SipPhone.prototype.__connect = function(data, callback) {
		var self = this;
		self.__init(function(errorThrown) {
			if (errorThrown) {
				callback(false, errorThrown.description);
				self.__emitError("Failed to initialize SIP stack: " + errorThrown.description, errorThrown);
			} else {
				callback(true);
				// create SIP stack
				data.events_listener.listener = _.bind(self.__stackEventListener, self);
				self.m_sipStack = new SIPml.Stack(data);

				// connect
				self.m_sipStack.start();
			}
		});
	};

	/*!
	 * Registers with the SIP proxy in order to make or receive calls
	 */
	SipPhone.prototype.__register = function() {
		// Create a REGISTER session in order to login
		this.logger.debug("SIPPhone: Register");
		this.m_regSession = this.m_sipStack.newSession('register', {
			expires         : 200,
			events_listener : {
				events  : '*',
				// use the same listener for all session
				listener: _.bind(this.__sessionEventListener, this)
			},
			sip_caps: [
				{ name: '+g.oma.sip-im', value: null },
				{ name: '+audio',        value: null },
				{ name: 'language',      value: '\"en,fr\"' }
			]
		});
		this.m_regSession.register();
	};

	SipPhone.prototype.__setState = function(state) {
		var previousState = this.currentState;
		this.logger.debug('SIPPhone: Change state = ' + state + ', previous = ' + previousState);
		this.currentState = state;
		return previousState;
	};

	SipPhone.prototype.__stackEventListener = function(e) { // SIPml.Stack.Event
		this.logger.debug('SIPPhone: Stack event = ' + e.type + ', current state: ' + this.currentState);
		var evt = { 
			type 			: e.type, 
			contentString  	: e.type === 'i_new_message' && e.getContentString() || null,
		};
		inConcert.AgentApi.StackEventNotification(evt);
		switch(e.type) {
			case 'started':
				this.__register();
				break;
			//case 'stopping':
			case 'stopped':
				if (this.currentState !== PhoneStates.IDLE) {
					this.__setState(PhoneStates.IDLE);
				}
				this.reconnect();
				break;
			case 'failed_to_start':
				this.__setState(PhoneStates.IDLE);
				this.reconnect();
				break;
			case 'failed_to_stop':
				this.__setState(PhoneStates.IDLE);
				break;
			case 'i_new_call':
				if (this.currentState !== PhoneStates.CONNECTED) {
					e.newSession.reject(); // cannot accept another call
				} else {
					this.__setState(PhoneStates.INBOUND);
					this.m_callSession = e.newSession;
					this.m_callSession.setConfiguration( this.__getCallConfig() );
				}
				break;
			case 'i_new_message':
				e.newSession.accept();
				break;
			case 'm_permission_requested':
				this.m_hasPermission = false;
				break;
			case 'm_permission_accepted':
			// this doesn't work too well in certain cases, mostly when permission request pile one above the other
			// we tried using a counter, but it doesn't always work
				this.m_hasPermission = true;
				break;
			case 'm_permission_refused':
				this.m_hasPermission = false;
				if (this.m_callSession) {
					this.m_callSession.hangup();
					this.m_callSession = null;
				}
				this.__setState(PhoneStates.CONNECTED);
				break;
			case 'starting':
				// acceptMessage(e);
		}
	};

	SipPhone.prototype.__sessionEventListener = function(e) { // SIPml.Session.Event
		console.info('SIPPhone: Session event = ' + e.type + ', current state: ' + this.currentState);
		var evt = { 
			type : e.type, 
		};
		switch(e.type) {
			case 'connecting':
				if (e.session === this.m_regSession) {
					evt.emit  = "connecting";
				} else if (e.session === this.m_callSession) {
					evt.emit  = "dialing";
				}
				inConcert.AgentApi.SessionEventNotification(evt);
				break;
			case 'connected':
				if (e.session === this.m_regSession) {
					this.__setState(PhoneStates.CONNECTED);
					evt.emit  = "connected";
				} else if (e.session === this.m_callSession) {
					this.__setState(PhoneStates.ACTIVE);
					evt.emit  = "accepted";
				}
				inConcert.AgentApi.SessionEventNotification(evt);
				break;
			case 'terminating':
				if (e.session === this.m_callSession && this.currentState === PhoneStates.OUTBOUND) {
					this.m_callSession = null;
					this.__setState(PhoneStates.CONNECTED);
					evt.emit  = "cancelled";
					inConcert.AgentApi.SessionEventNotification(evt);
					break;
				}
				/*falls through*/
			case 'terminated':
				var previousState;
				if (e.session === this.m_regSession) {
					this.m_callSession = null;
					this.m_regSession = null;
					previousState = this.__setState(PhoneStates.IDLE);
					evt.emit  = "idle";
					inConcert.AgentApi.SessionEventNotification(evt);
					//if (previousState !== PhoneStates.IDLE) {
						//this.__emitError("Connection terminated");
					//}
					this.reconnect();
				} else if (e.session === this.m_callSession) {
					this.m_callSession = null;
					previousState = this.__setState(PhoneStates.CONNECTED);
					switch(previousState) {
						case PhoneStates.OUTBOUND:
							evt.emit  = "rejected";
							inConcert.AgentApi.SessionEventNotification(evt);
							break;
						case PhoneStates.INBOUND:
						case PhoneStates.ACTIVE:
							evt.emit  = "byed";
							inConcert.AgentApi.SessionEventNotification(evt);
							break;
						default:
							this.logger.warn("SIPPhone: Session terminated while in state " + previousState);
							break;
					}
					if (!evt.emit) {
						evt.emit  = "connected";
						inConcert.AgentApi.SessionEventNotification(evt);
					}
				}
				break;
			case 'm_stream_video_local_added':
			case 'm_stream_video_local_removed':
			case 'm_stream_video_remote_added':
			case 'm_stream_video_remote_removed':
				break;
			case 'm_stream_audio_local_added':
			case 'm_stream_audio_local_removed':
			case 'm_stream_audio_remote_added':
			case 'm_stream_audio_remote_removed':
				break;
			case 'i_ect_new_call':
				break;
			case 'i_ao_request':
				if (e.session === this.m_callSession) {
					var iSipResponseCode = e.getSipResponseCode();
					console.info('SIPPhone: i_ao_request response = ' + iSipResponseCode);
					if (iSipResponseCode === 180 || iSipResponseCode === 183) {				
						evt.emit  = "ringing";
						inConcert.AgentApi.SessionEventNotification(evt);
					}
				}
				break;
			case 'm_early_media':
				break;
			case 'm_local_hold_ok':
				break;
			case 'm_local_hold_nok':
				break;
			case 'm_local_resume_ok':
				break;
			case 'm_local_resume_nok':
				break;
			case 'm_remote_hold':
				if (e.session === this.m_callSession) {
					evt.emit  = "remoteHold";
					inConcert.AgentApi.SessionEventNotification(evt);
				}
				break;
			case 'm_remote_resume':
				if (e.session === this.m_callSession) {
					evt.emit  = "remoteResume";
					inConcert.AgentApi.SessionEventNotification(evt);
				}
				break;
			case 'o_ect_trying':
				break;
			case 'o_ect_accepted':
				break;
			case 'o_ect_completed':
			case 'i_ect_completed':
				break;
			case 'o_ect_failed':
			case 'i_ect_failed':
				break;
			case 'o_ect_notify':
			case 'i_ect_notify':
				break;
			case 'i_ect_requested':
				break;
		}
	};
	
	SipPhone.prototype.getPhoneState = function() {
		return this.currentState;
	};

	if (!inConcert.SipPhone) {
		inConcert.SipPhone = SipPhone;
		inConcert.PhoneStates = PhoneStates;
	}
}());
