"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const logger_config_1 = require("../config/logger.config");
const mssql_service_1 = require("../db/mssql.service");
const mssql_config_1 = require("../config/mssql.config");
const mssql_init_1 = require("../db/mssql.init");
const redis_init_1 = require("../db/redis.init");
const mail_init_1 = require("../mail/mail.init");
const redis_service_1 = require("../db/redis.service");
const log4js = require("log4js");
const promises_shared_1 = require("../shared/promises.shared");
const find_process_task_1 = require("./find-process.task");
const file_manager_1 = require("../shared/file-manager");
const config_1 = require("../config/config");
const smtp_mail_1 = require("../mail/smtp.mail");
const node_ssh_1 = require("node-ssh");
log4js.configure(logger_config_1.LOGGER_CONFIG);
const logger = log4js.getLogger('Cleaner');
const RETRY = 3;
const ITEMS_PER_PAGE = 5;
const ROOT_KEY = "CleanerTaskData";
const IDATHA_SCRIPT = 'jobs/realm_job.sh';
const EMAILS = "amaravi@inconcertcc.com";
const PATHS = [
    config_1.SEMANCTIC_ANALYSIS_VIEW_FOLDER,
    config_1.TRANSCRIPTION_FOLDER,
    config_1.PATH_FOLDER_TEMP_NLU
];
find_process_task_1.ProcessInRunning().then((processRunning) => {
    if (processRunning) {
        logger.info('[cleaner.task::Run] Found process running');
        return Promise.resolve(false);
    }
    return Run();
}).then((result) => {
    logger.info('[cleaner.task::Run] Finished Cleaner Task:', result);
});
function Run() {
    logger.info('[cleaner.task::Run] Begin Cleaner Task');
    mssql_config_1.MSSQL_CONFIG.connectionTimeout = 300 * 1000;
    mssql_config_1.MSSQL_CONFIG.requestTimeout = 300 * 1000;
    let app = express();
    let sqlConn;
    let redisClient;
    let mailServer;
    let isConnectedEmailServer = false;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return redis_init_1.RedisInit(app);
    })
        .then(_ => {
        redisClient = app.get('redisClient');
        return true;
    })
        .then(_ => {
        return mssql_init_1.MsSqlInit(app);
    })
        .then(_ => {
        sqlConn = app.get('sqlConn');
        return true;
    })
        .then(_ => {
        return mail_init_1.MailInit(app);
    })
        .then((mailConnected) => {
        mailServer = app.get('mailserver');
        isConnectedEmailServer = mailConnected;
        return true;
    })
        .then((_) => {
        return mssql_service_1.MSSql.GetVccCleanerConfig();
    })
        .then((vccs) => {
        if (typeof vccs == 'object' && vccs.length > 0) {
            logger.info('[cleaner.task::Run] Found vccs:', vccs.length);
            let promises = [
                Promise.resolve(true),
                Promise.resolve(vccs)
            ];
            for (let row of vccs) {
                let loader = new DataLoader(row.vcc, row.deleteDataDays);
                promises.push(loader.Run());
            }
            return Promise.all(promises);
        }
        logger.debug('[cleaner.task::Run] Not found vccs');
        return [
            false
        ];
    })
        .then((results) => {
        let promises = [];
        let foundData = results[0];
        if (foundData) {
            let vccs = results[1];
            promises.push(DatabaseCleanerHandler());
            promises.push(FileCleanerHandler(vccs));
            promises.push(IdathaCleanerHandler(vccs));
        }
        else {
            promises.push(Promise.resolve(false));
        }
        return Promise.all(promises);
    })
        .then((results) => {
        logger.info('[cleaner.task::Run] Tasks cleanners result:', results);
        if (isConnectedEmailServer) {
            let emailSender = new EmailSender();
            return emailSender.Run();
        }
        return true;
    })
        .then((_) => {
        return DeleteAllTasks();
    })
        .then((result) => {
        sqlConn.close();
        redisClient.quit();
        mailServer.close();
        return result;
    })
        .catch(err => {
        logger.error('[cleaner.task::Run] Error:', err);
        if (sqlConn) {
            sqlConn.close();
        }
        if (redisClient) {
            redisClient.quit();
        }
        if (mailServer) {
            mailServer.close();
        }
        return false;
    });
}
class DataLoader {
    constructor(vcc, days) {
        this.vcc = vcc;
        this.days = days;
    }
    Run() {
        return mssql_service_1.MSSql.GetCleanerData(this.vcc, this.days)
            .then((results) => {
            return this.ProcessData(results);
        }).catch(err => {
            logger.error('[cleaner.task::DataLoader->Run] Error:', err);
            return false;
        });
    }
    ProcessData(rows) {
        if (rows && rows.length > 0) {
            let evaluationPlans = rows[0];
            let studies = rows[1];
            let process = rows[2];
            let processAsyncs = rows[3];
            let audios = rows[4];
            let asyncInteractions = rows[5];
            let promises = [];
            logger.info(`[cleaner.task::DataLoader->ProcessData] Result for ${this.vcc} with days ${this.days}:`, JSON.stringify({
                plan: evaluationPlans.length,
                study: studies.length,
                process: process.length,
                processAsyncs: processAsyncs.length,
                audios: audios.length,
                asyncInteractions: asyncInteractions.length
            }));
            if (evaluationPlans.length > 0) {
                promises.push(this.SaveToRedis(evaluationPlans, 'evaluationPlans', 'id'));
            }
            if (studies.length > 0) {
                promises.push(this.SaveToRedis(studies, 'studies', 'name'));
            }
            if (process.length > 0) {
                promises.push(this.SaveToRedis(process, 'process', 'name'));
            }
            if (processAsyncs.length > 0) {
                promises.push(this.SaveToRedis(processAsyncs, 'processAsyncs', 'processId'));
            }
            if (audios.length > 0) {
                promises.push(this.SaveToRedis(audios, 'audios', 'id', {
                    retry: 0
                }));
            }
            if (asyncInteractions.length > 0) {
                promises.push(this.SaveToRedis(asyncInteractions, 'asyncInteractions', 'id'));
            }
            return Promise.all(promises).then((results) => {
                return !results.includes(false);
            });
        }
        return Promise.resolve(false);
    }
    SaveToRedis(rows, key, id, mixin = {}) {
        return redis_service_1.Redis.SaveCleanerData(`${ROOT_KEY}:${key}`, id, this.vcc, mixin, rows);
    }
}
class AudioCleaner {
    Run() {
        logger.info('[cleaner.task::AudioCleaner->Run] Begin read audio cleaner');
        return promises_shared_1.DummyPromise().then(_ => {
            return this.Read(0, ITEMS_PER_PAGE);
        });
    }
    Read(start, end) {
        return promises_shared_1.DummyPromise().then(_ => {
            return redis_service_1.Redis.LRange(`${ROOT_KEY}:audios:Keys`, start, end - 1);
        }).then((keys) => {
            let promises = [];
            if (keys && keys.length > 0) {
                logger.info('[cleaner.task::AudioCleaner->Read] Found audio keys:', keys.length);
                promises.push(Promise.resolve(true));
                promises.push(redis_service_1.Redis.GetCleanerData(`${ROOT_KEY}:audios`, keys));
            }
            else {
                promises.push(Promise.resolve(false));
                logger.debug('[cleaner.task::AudioCleaner->Read] Not found audio keys:', keys);
            }
            return Promise.all(promises);
        }).then((results) => {
            let found = results[0];
            let promises = [Promise.resolve(found)];
            if (found) {
                let _audios = results[1];
                for (let a of _audios) {
                    let audio = JSON.parse(a);
                    promises.push(this.Clean(audio));
                }
            }
            return Promise.all(promises);
        }).then((results) => {
            if (results[0]) {
                return this.Read(start + ITEMS_PER_PAGE, end + ITEMS_PER_PAGE);
            }
            return true;
        });
    }
    Clean(audio, message = '') {
        let path = `${config_1.PATH_FOLDER_PROCESS_AUDIO_PLAY}/${audio.vcc}/${audio.fileName}`;
        logger.info('[cleaner.task::AudioCleaner->Clean] Deleting audio:', path);
        if (audio.retry < RETRY) {
            return promises_shared_1.DummyPromise().then(_ => {
                if (audio.fileName) {
                    return file_manager_1.ExistInFileSystem(path);
                }
                return false;
            })
                .then((exists) => {
                if (exists) {
                    return file_manager_1.DeleteFile(path);
                }
                return true;
            })
                .then((deleted) => {
                logger.info(`[cleaner.task::AudioCleaner->Clean] Deleted audio ${path}:`, deleted);
                let promises = [Promise.resolve(deleted)];
                if (deleted) {
                    promises.push(redis_service_1.Redis.DeleteCleanerData(`${ROOT_KEY}:audios`, `${audio.vcc}:${audio.id}`));
                    promises.push(mssql_service_1.MSSql.DeleteInteraction(audio.vcc, audio.id, true));
                }
                return Promise.all(promises);
            }).then((results) => {
                let deleted = results[0];
                if (!deleted) {
                    audio.retry += 1;
                    return this.Clean(Object.assign({}, audio), `Error desconocido al eliminar el audio ${audio.id}`);
                }
                return !results.includes(false);
            }).catch(err => {
                logger.error(`[cleaner.task::AudioCleaner->Clean] Error on delete audio ${audio.id} for ${audio.vcc}:`, err);
                audio.retry += 1;
                return this.Clean(Object.assign({}, audio), getErrorMessage(err));
            });
        }
        return SaveCleanerError({
            vcc: audio.vcc,
            description: message,
            raw: JSON.stringify(audio),
            process: 'Speech binarios [audio]'
        });
    }
}
function DatabaseCleanerHandler() {
    logger.info('[cleaner.task::DatabaseCleanerHandler] Begin database cleaner');
    let promises = [];
    let evaluationPlans = new DatabaseCleaner('evaluationPlans', 'id', 1);
    let studies = new DatabaseCleaner('studies', 'name', 2);
    let process = new DatabaseCleaner('process', 'name', 3);
    let processAsyncs = new DatabaseCleaner('processAsyncs', 'processId', 4);
    let asyncInteractions = new DatabaseCleaner('asyncInteractions', 'id', 5);
    let audioCleaner = new AudioCleaner();
    promises.push(evaluationPlans.Run());
    promises.push(studies.Run());
    promises.push(process.Run());
    promises.push(processAsyncs.Run());
    promises.push(asyncInteractions.Run());
    promises.push(audioCleaner.Run());
    return promises_shared_1.DummyPromise().then(_ => {
        return Promise.all(promises);
    })
        .then((results) => {
        let description = '';
        let result = !results.includes(false);
        if (result) {
            description = 'Finalizado correctamente';
        }
        else {
            description = 'Finalizado con errores';
        }
        return SaveCleanerSummary('speech_database', result, description);
    }).catch(err => {
        logger.error(`[cleaner.task::DatabaseCleanerHandler] Error on clear in speech database:`, err);
        return SaveCleanerSummary('speech_database', false, getErrorMessage(err));
    });
}
class DatabaseCleaner {
    constructor(key, idKey, type) {
        this.key = key;
        this.idKey = idKey;
        this.type = type;
    }
    Run() {
        logger.info('[cleaner.task::DatabaseCleaner->Run] Begin database cleaner for:', this.key);
        return promises_shared_1.DummyPromise().then(_ => {
            return this.Read(0, ITEMS_PER_PAGE);
        });
    }
    Read(start, end) {
        return promises_shared_1.DummyPromise().then(_ => {
            return redis_service_1.Redis.LRange(`${ROOT_KEY}:${this.key}:Keys`, start, end - 1);
        }).then((keys) => {
            let promises = [];
            if (keys && keys.length > 0) {
                logger.info(`[cleaner.task::DatabaseCleaner->Read] Found ${this.key} keys:`, keys.length);
                promises.push(Promise.resolve(true));
                promises.push(redis_service_1.Redis.GetCleanerData(`${ROOT_KEY}:${this.key}`, keys));
            }
            else {
                promises.push(Promise.resolve(false));
                logger.debug(`[cleaner.task::DatabaseCleaner->Read] Not found ${this.key} keys:`, keys);
            }
            return Promise.all(promises);
        }).then((results) => {
            let found = results[0];
            let promises = [Promise.resolve(found)];
            if (found) {
                let _items = results[1];
                for (let i of _items) {
                    let item = JSON.parse(i);
                    if (item) {
                        promises.push(this.Clean(item));
                    }
                }
            }
            return Promise.all(promises);
        }).then((results) => {
            if (results[0]) {
                return this.Read(start + ITEMS_PER_PAGE, end + ITEMS_PER_PAGE);
            }
            return true;
        }).catch(err => {
            logger.info(`[cleaner.task::DatabaseCleaner->Read] Error on read ${this.key}`, err);
            return false;
        });
    }
    Clean(item, retry = 0, message = '') {
        logger.info(`[cleaner.task::DatabaseCleaner->Clean] Deleting ${this.key}:`, item[this.idKey]);
        if (retry < RETRY) {
            return promises_shared_1.DummyPromise().then(_ => {
                switch (this.type) {
                    case 1:
                        return mssql_service_1.MSSql.DeleteEvaluationPlan(item.vcc, item[this.idKey]);
                    case 2:
                        return mssql_service_1.MSSql.DeleteStudy(item.vcc, item[this.idKey]);
                    case 3:
                        return mssql_service_1.MSSql.DeleteProcess(item.vcc, item[this.idKey]);
                    case 4:
                        return mssql_service_1.MSSql.DeleteProcessAsync(item.vcc, item[this.idKey]);
                    case 5:
                        return mssql_service_1.MSSql.DeleteInteraction(item.vcc, item[this.idKey], false);
                    default: break;
                }
                return false;
            }).then((deleted) => {
                logger.info(`[cleaner.task::DatabaseCleaner->Clean] Deleted ${this.key} ${item[this.idKey]}:`, deleted);
                let promises = [Promise.resolve(deleted)];
                if (deleted) {
                    promises.push(redis_service_1.Redis.DeleteCleanerData(`${ROOT_KEY}:${this.key}`, `${item.vcc}:${item[this.idKey]}`));
                }
                return Promise.all(promises);
            }).then((results) => {
                let deleted = results[0];
                if (!deleted) {
                    return this.Clean(item, retry + 1, `Error desconocido al eliminar ${this.key} : ${item[this.idKey]}`);
                }
                return !results.includes(false);
            }).catch(err => {
                logger.error(`[cleaner.task::DatabaseCleaner->Clean] Error on delete ${this.key} ${item[this.idKey]} for ${item.vcc}:`, err);
                return this.Clean(item, retry + 1, getErrorMessage(err));
            });
        }
        return SaveCleanerError({
            vcc: item.vcc,
            description: message,
            raw: JSON.stringify(item),
            process: 'Speech base de datos'
        });
    }
}
class EmailSender {
    Run() {
        logger.info(`[cleaner.task::EmailSender->Run] Begin`);
        return promises_shared_1.DummyPromise().then(_ => {
            return redis_service_1.Redis.LRange(`${ROOT_KEY}:errors`, 0, -1);
        }).then((results) => {
            if (results && results.length > 0) {
                logger.info(`[cleaner.task::EmailSender->Run] Found errors:`, results.length);
                return this.StartSend(results);
            }
            logger.info(`[cleaner.task::EmailSender->Run] Not found errors`);
            return true;
        });
    }
    StartSend(rows) {
        return promises_shared_1.DummyPromise().then(_ => {
            return mssql_service_1.MSSql.GetVersion();
        }).then((version) => {
            logger.info(`[cleaner.task::EmailSender->StartSend] installationId:`, version.installationId);
            return this.Send(rows, version.installationId);
        }).catch((err) => {
            logger.error(`[cleaner.task::EmailSender->StartSend] Error on get installationId and send email:`, err);
            return false;
        });
    }
    Send(rows, installationId) {
        let errors = [];
        for (let row of rows) {
            let error = JSON.parse(row);
            errors.push(`
           <tr>
           <td>${error.vcc}</td>
           <td>${error.process}</td>
           <td>${error.description}</td>
           <td>${error.raw}</td>
           </tr>
           `);
        }
        logger.info(`[cleaner.task::EmailSender->Send] Send email`);
        let template = `
        <h3>Hola, el proceso de eliminaci&oacute;n ha finalizado con errores</h3>
        <p>Los siguientes objetos no se han eliminado:</p>
        <br>
        <table border="1">
      <thead>
      <tr>
        <th>VCC</th>
        <th>Proceso</th>
        <th>Descripci&oacute;n</th>
        <th>Valor(raw)</th>
        </tr>
      </thead> 
      <tbody>
      ${errors.join(' ')}
      </tbody> 
        </table>
        `;
        return promises_shared_1.DummyPromise().then(_ => {
            return smtp_mail_1.Send(EMAILS, `Error de eliminación en el ambiente ${installationId}`, template);
        }).then((success) => {
            logger.info(`[cleaner.task::EmailSender->Send] Result send email:`, success);
            return success;
        }).catch((err) => {
            logger.error(`[cleaner.task::EmailSender->Send] Error on send email:`, err);
            return false;
        });
    }
}
function FileCleanerHandler(vccs) {
    let promises = [];
    for (let row of vccs) {
        let fileCleaner = new FileCleaner(row.vcc, row.deleteDataDays);
        promises.push(fileCleaner.Run());
    }
    return promises_shared_1.DummyPromise().then(_ => {
        return Promise.all(promises);
    }).then((results) => {
        let description = '';
        let result = !results.includes(false);
        if (result) {
            description = 'Finalizado correctamente';
        }
        else {
            description = 'Finalizado con errores';
        }
        return SaveCleanerSummary('speech_files', result, description);
    }).catch(err => {
        logger.error(`[cleaner.task::FileCleanerHandler] Error on clear files:`, err);
        return SaveCleanerSummary('speech_files', false, getErrorMessage(err));
    });
}
class FileCleaner {
    constructor(vcc, days) {
        this.vcc = vcc;
        this.days = days;
        this.seconds = this.days * 24 * 60 * 60;
    }
    Run() {
        let promises = [];
        for (let path of PATHS) {
            promises.push(this.Clean(`${path}/${this.vcc}`));
        }
        return Promise.all(promises).then((results) => {
            logger.debug(`[cleaner.task::FileCleaner->Run] Clean results for ${this.vcc}:`, results);
            return !results.includes(false);
        });
    }
    Clean(path, retry = 0, message = '') {
        logger.info(`[cleaner.task::FileCleaner->Clean] Deleting path:`, path);
        if (retry < RETRY) {
            return promises_shared_1.DummyPromise().then(_ => {
                return file_manager_1.ExistInFileSystem(path);
            }).then((found) => {
                let promises = [Promise.resolve(found)];
                if (found) {
                    logger.info(`[cleaner.task::FileCleaner->Clean] Found path ${path}`);
                    promises.push(file_manager_1.CleanupTempFile(path, this.seconds));
                }
                logger.info(`[cleaner.task::FileCleaner->Clean] Not found path ${path}`);
                return Promise.all(promises);
            })
                .then((results) => {
                let found = results[1];
                if (found) {
                    let deleted = results[1];
                    logger.info(`[cleaner.task::FileCleaner->Clean] Deleted path ${path}:`, deleted);
                    if (!deleted) {
                        return this.Clean(path, retry + 1, `Error desconocido al eliminar la carpeta ${path}`);
                    }
                    return deleted;
                }
                return true;
            })
                .catch(err => {
                logger.error(`[cleaner.task::FileCleaner->Clean] Error on delete path ${path}:`, err);
                return this.Clean(path, retry + 1, getErrorMessage(err));
                ;
            });
        }
        return SaveCleanerError({
            vcc: this.vcc,
            description: message,
            raw: '',
            process: 'Speech binarios'
        });
    }
}
function IdathaCleanerHandler(vccs) {
    if (vccs.length == 0) {
        return Promise.resolve(true);
    }
    return promises_shared_1.DummyPromise().then(_ => {
        let row = vccs.pop();
        let idathaCleaner = new IdathaCleaner(row.vcc, row.deleteDataDays);
        return idathaCleaner.Run();
    })
        .then((result) => {
        let description = '';
        if (result) {
            description = 'Finalizado correctamente';
        }
        else {
            description = 'Finalizado con errores';
        }
        return SaveCleanerSummary('idatha', result, description);
    })
        .then((_) => {
        return IdathaCleanerHandler(vccs);
    })
        .catch(err => {
        logger.error(`[cleaner.task::IdathaCleanerHandler] Error on clear in idatha:`, err);
        return SaveCleanerSummary('idatha', false, getErrorMessage(err));
    });
}
class IdathaCleaner {
    constructor(vcc, days) {
        this.vcc = vcc;
        this.days = days;
        let d = new Date();
        let add = 0;
        if (this.days < 90) {
            add = 90;
        }
        d.setDate(d.getDate() - (this.days + add));
        this.date = d.toISOString().substring(0, 10);
    }
    Run(retry = 0, message = '') {
        let cmd = `${IDATHA_SCRIPT} ${this.date} ${this.vcc}`;
        if (retry < RETRY) {
            let ssh = new node_ssh_1.NodeSSH();
            return promises_shared_1.DummyPromise().then(_ => {
                return ssh.connect({
                    host: config_1.IDATHA_IP,
                    username: config_1.IDATHA_USER,
                    password: config_1.IDATHA_PASSWORD,
                    port: config_1.IDATHA_SSH_PORT,
                    readyTimeout: 99999
                });
            }).then((_) => {
                if (ssh.isConnected) {
                    logger.info('[cleaner.task::IdathaCleaner->Run] Run command in idatha:', cmd);
                    return ssh.execCommand(cmd, { cwd: '/home/admon/in_concert_speech_analytics/' });
                }
                logger.error('[cleaner.task::IdathaCleaner->Run] Not connected to idatha');
                return null;
            })
                .then((response) => {
                ssh.dispose();
                if (typeof response == 'object') {
                    if (response.stdout) {
                        try {
                            let output = JSON.parse(response.stdout);
                            logger.debug(`[cleaner.task::IdathaCleaner->Run] Output for command ${cmd}:`, output);
                            if (typeof output.deleted_data == 'object') {
                                return { state: true };
                            }
                            return { state: false, message: `Error desconocido al eliminar el realm ${this.vcc}, stdout is : ${JSON.stringify(output)}` };
                        }
                        catch (err) {
                            return { state: false, message: getErrorMessage(err) };
                        }
                    }
                    return { state: false, message: response.stderr };
                }
                return { state: false, message: 'Connection fail' };
            })
                .then((result) => {
                if (!result.state) {
                    return this.Run(retry + 1, result.message);
                }
                return result.state;
            })
                .catch((error) => {
                ssh.dispose();
                let error_message = getErrorMessage(error);
                logger.error('[cleaner.task::IdathaCleaner->Run] Error on connect to idatha:', error_message);
                return this.Run(retry + 1, error_message);
            });
        }
        return SaveCleanerError({
            vcc: this.vcc,
            description: message,
            raw: cmd,
            process: 'idatha'
        });
    }
}
function SaveCleanerError(data, returnValue = false) {
    logger.debug('[cleaner.task::SaveCleanerError] Saving error for:', data.vcc);
    return promises_shared_1.DummyPromise().then(_ => {
        return redis_service_1.Redis.SaveCleanerError(`${ROOT_KEY}:errors`, data);
    }).then(_ => {
        return returnValue;
    });
}
function SaveCleanerSummary(task, status, description) {
    logger.debug('[cleaner.task::SaveCleanerSummary] Saving summary for task:', task);
    return mssql_service_1.MSSql.SaveCleanerSummary(task, status, description.length > 250 ? description.substring(0, 250) : description);
}
function DeleteAllTasks() {
    let tasks = [
        `${ROOT_KEY}:evaluationPlans`,
        `${ROOT_KEY}:evaluationPlans:Keys`,
        `${ROOT_KEY}:studies`,
        `${ROOT_KEY}:studies:Keys`,
        `${ROOT_KEY}:asyncInteractions`,
        `${ROOT_KEY}:asyncInteractions:Keys`,
        `${ROOT_KEY}:audios`,
        `${ROOT_KEY}:audios:Keys`,
        `${ROOT_KEY}:process`,
        `${ROOT_KEY}:process:Keys`,
        `${ROOT_KEY}:processAsyncs`,
        `${ROOT_KEY}:processAsyncs:Keys`,
        `${ROOT_KEY}:errors`,
    ];
    return redis_service_1.Redis.DeleteAllCleanerTasks(tasks).then((result) => {
        logger.info('[cleaner.task::DeleteAllTasks] Result for delete redis task:', result);
        return true;
    }).catch((err) => {
        logger.error('[cleaner.task::DeleteAllTasks] Error on clear redis tasks:', err);
        return false;
    });
}
function getErrorMessage(err) {
    return (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
}
